# -*- coding: utf-8 -*-
import logging

from puzzle.models.sql import Resource

logger = logging.getLogger(__name__)


class ResourceActions(object):
    def add_resource(self, name, file_path, ind_obj):
        """Link a resource to an individual."""
        new_resource = Resource(name=name, individual=ind_obj, path=file_path)
        self.session.add(new_resource)
        self.save()
        return new_resource

    def resource(self, resource_id):
        """Fetch a resource."""
        return self.query(Resource).get(resource_id)

    def delete_resource(self, resource_id):
        """Link a resource to an individual."""
        resource_obj = self.resource(resource_id)
        logger.debug("Deleting resource {0}".format(resource_obj.name))
        self.session.delete(resource_obj)
        self.save()



import sys
import logging
import subprocess

from pprint import pprint as pp
from datetime import datetime
import click

@click.command()
@click.argument('vcf',
    nargs=1,
    type=click.Path(exists=True),
    # type=click.Path(exists=True),
)
@click.option('-v', '--variant',
    nargs=1,
    type=int
)
def test_grep(vcf, variant):
    """Test grep"""
    if vcf.endswith('.gz'):
        cat = subprocess.Popen(['zcat', vcf], 
                        stdout=subprocess.PIPE,
                        )
    else:
        cat = subprocess.Popen(['cat', vcf], 
                                stdout=subprocess.PIPE,
                                )

    grep_commands = ["grep", "BND\|DEL"]
    grep_commands = ["grep", "BND"]
    
    grep = subprocess.Popen(grep_commands,
                        stdin=cat.stdout,
                        stdout=subprocess.PIPE,
                        )
    
    end_of_pipe = grep.stdout
    
    for line in end_of_pipe:
        if not line.startswith('#'):
            print(line.split('\t'))

    


if __name__ == '__main__':
    test_grep()



# -*- coding: utf-8 -*-
from sqlalchemy import (Column, Integer, String)

from .models import BASE


class GeminiQuery(BASE):

    """Represent a user defined gemini query."""

    __tablename__ = "gemini_query"

    id = Column(Integer, primary_key=True)
    name = Column(String(32), nullable=False, unique=True)
    query = Column(String(128))

    @property
    def name_query(self):
        return self.name or self.query

    def __repr__(self):
        return ("GeminiQuery(name={this.name), query={this.query}"
                .format(this=self))



# -*- coding: utf-8 -*-


class DotDict(dict):
    """
    Example:
    test = DotDict(last_name='Pool', age=24, sports=['Soccer'])
    """
    def __init__(self, *args, **kwargs):
        super(DotDict, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for key, value in arg.iteritems():
                    self[key] = value

        if kwargs:
            for key, value in kwargs.iteritems():
                self[key] = value

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(DotDict, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(DotDict, self).__delitem__(key)
        del self.__dict__[key]



# -*- coding: utf-8 -*-
from puzzle.plugins import VcfPlugin

def test_add_compounds(variant):
    plugin = VcfPlugin()
    info_dict = {
        'Compounds': "643594:11_120999413_G_A>24|11_120988038_T_G>23|11_120991571_G_A>10|11_121028581_C_G>36"
    }
    plugin._add_compounds(variant, info_dict)
    compounds = variant.compounds
    highest_scored_compound = compounds[0]
    assert int(highest_scored_compound.combined_score) == 36

def test_add_compounds_no_compounds(variant):
    plugin = VcfPlugin()
    info_dict = {}
    plugin._add_compounds(variant, info_dict)
    assert variant.compounds == []

def test_add_cadd_score(variant):
    plugin = VcfPlugin()
    info_dict = {'CADD': '24'}
    plugin._add_cadd_score(variant, info_dict)
    
    assert float(variant.cadd_score) == 24.0

def test_add_cadd_score_no_score(variant):
    plugin = VcfPlugin()
    info_dict = {}
    plugin._add_cadd_score(variant, info_dict)
    
    assert variant.cadd_score == None

def test_add_genetic_models(variant):
    plugin = VcfPlugin()
    info_dict = {'GeneticModels': '643594:AD_dn|AR_comp_dn'}
    plugin._add_genetic_models(variant, info_dict)
    
    assert set(variant.genetic_models) == set(['AD_dn', 'AR_comp_dn'])

def test_add_genetic_models_no_models(variant):
    plugin = VcfPlugin()
    info_dict = {}
    plugin._add_genetic_models(variant, info_dict)
    
    assert variant.genetic_models == []

def test_add_rank_score(variant):
    plugin = VcfPlugin()
    info_dict = {'RankScore': '643594:24'}
    plugin._add_rank_score(variant, info_dict)
    
    assert int(variant.rank_score) == 24

def test_add_rank_score_no_score(variant):
    plugin = VcfPlugin()
    info_dict = {}
    plugin._add_rank_score(variant, info_dict)
    
    assert variant.rank_score == None




class BaseCaseMixin(object):
    "Base plugin for caase mixins"
    
    def cases(self, pattern=None):
        """Return all cases."""
        raise NotImplementedError

    def case(self, case_id=None):
        """Return a case."""
        raise NotImplementedError

    def add_case(self, ind_id=None):
        """Return a individual."""
        raise NotImplementedError

    def individual(self, ind_id=None):
        """Return a individual."""
        raise NotImplementedError

    def individuals(self, ind_id=None):
        """Return a individual."""
        raise NotImplementedError

    


from puzzle.models.sql import Case, Individual, BASE

def test_case(session):
    """Test to add a case to the session"""
    # proband = Individual(ind_id="Proband")
    family = Case(case_id="afamily")
    # proband.case = family
    session.add(family)
    session.commit()

    fam = session.query(Case).filter(Case.case_id == "afamily").one()
    assert fam.id == 1

def test_individual(session):
    """Test to add a case to the session"""
    # proband = Individual(ind_id="Proband")
    family = Case(case_id="2")
    proband = Individual(ind_id="Proband")
    proband.cases.append(family)
    session.add(proband)
    session.add(family)
    session.commit()

    fam = session.query(Case).filter(Case.case_id == "2").one()
    assert fam.individuals[0].ind_id == "Proband"
    


# -*- coding: utf-8 -*-
import logging

import phizz

from puzzle.utils import hpo_genes
from puzzle.models.sql import PhenotypeTerm

logger = logging.getLogger(__name__)


class PhenotypeActions(object):
    def add_phenotype(self, ind_obj, phenotype_id):
        """Add a phenotype term to the case."""
        if phenotype_id.startswith('HP:') or len(phenotype_id) == 7:
            logger.debug('querying on HPO term')
            hpo_results = phizz.query_hpo([phenotype_id])
        else:
            logger.debug('querying on OMIM term')
            hpo_results = phizz.query_disease([phenotype_id])

        added_terms = [] if hpo_results else None
        existing_ids = set(term.phenotype_id for term in ind_obj.phenotypes)
        for result in hpo_results:
            if result['hpo_term'] not in existing_ids:
                term = PhenotypeTerm(phenotype_id=result['hpo_term'],
                                     description=result['description'])
                logger.info('adding new HPO term: %s', term.phenotype_id)
                ind_obj.phenotypes.append(term)
                added_terms.append(term)

        logger.debug('storing new HPO terms')
        self.save()

        if added_terms is not None and len(added_terms) > 0:
            for case_obj in ind_obj.cases:
                self.update_hpolist(case_obj)

        return added_terms

    def update_hpolist(self, case_obj):
        """Update the HPO gene list for a case based on current terms."""
        hpo_list = self.case_genelist(case_obj)
        hpo_results = hpo_genes(case_obj.phenotype_ids(),
                                *self.phenomizer_auth)

        if hpo_results is None:
            pass
            # Why raise here?
            # raise RuntimeError("couldn't link to genes, try again")
        else:
            gene_ids = [result['gene_id'] for result in hpo_results
                        if result['gene_id']]
            hpo_list.gene_ids = gene_ids
            self.save()

    def remove_phenotype(self, ind_obj, phenotypes=None):
        """Remove multiple phenotypes from an individual."""
        if phenotypes is None:
            logger.info("delete all phenotypes related to %s", ind_obj.ind_id)
            self.query(PhenotypeTerm).filter_by(ind_id=ind_obj.id).delete()
        else:
            for term in ind_obj.phenotypes:
                if term.phenotype_id in phenotypes:
                    logger.info("delete phenotype: %s from %s",
                                term.phenotype_id, ind_obj.ind_id)
                    self.session.delete(term)
        logger.debug('persist removals')
        self.save()
        for case_obj in ind_obj.cases:
            self.update_hpolist(case_obj)



# -*- coding: utf-8 -*-
import os

SV_TYPES = (
    'DEL',
    'DUP',
    'INV',
    'INS',
    'CNV',
    'BND',
    'TDUP',
)


INHERITANCE_MODELS = (
    ('AR_hom', 'Autosomal Recessive Homozygote'),
    ('AR_hom_dn', 'Autosomal Recessive Homozygote De Novo'),
    ('AR_comp', 'Autosomal Recessive Compound'),
    ('AR_comp_dn', 'Autosomal Recessive Compound De Novo'),
    ('AD', 'Autosomal Dominant'),
    ('AD_dn', 'Autosomal Dominant De Novo'),
    ('XR', 'X Linked Recessive'),
    ('XR_dn', 'X Linked Recessive De Novo'),
    ('XD', 'X Linked Dominant'),
    ('XD_dn', 'X Linked Dominant De Novo'),
)

INHERITANCE_MODELS_SHORT = [model[0] for model in INHERITANCE_MODELS]

SO_TERMS = (
    'transcript_ablation',
    'splice_donor_variant',
    'splice_acceptor_variant',
    'stop_gained',
    'frameshift_variant',
    'stop_lost',
    'start_lost',
    'initiator_codon_variant',
    'transcript_amplification',
    'inframe_insertion',
    'inframe_deletion',
    'missense_variant',
    'protein_altering_variant',
    'splice_region_variant',
    'incomplete_terminal_codon_variant',
    'stop_retained_variant',
    'synonymous_variant',
    'coding_sequence_variant',
    'mature_miRNA_variant',
    '5_prime_UTR_variant',
    '3_prime_UTR_variant',
    'non_coding_exon_variant',
    'non_coding_transcript_exon_variant',
    'non_coding_transcript_variant',
    'nc_transcript_variant',
    'intron_variant',
    'NMD_transcript_variant',
    'non_coding_transcript_variant',
    'upstream_gene_variant',
    'downstream_gene_variant',
    'TFBS_ablation',
    'TFBS_amplification',
    'TF_binding_site_variant',
    'regulatory_region_ablation',
    'regulatory_region_amplification',
    'regulatory_region_variant',
    'feature_elongation',
    'feature_truncation',
    'intergenic_variant'
)

IMPACT_LEVELS = (
    'HIGH',
    'MEDIUM',
    'LOW',
)

PUZZLE_CONFIG_PATH = os.path.expanduser("~/.puzzlerc")


# -*- coding: utf-8 -*-
import logging

from puzzle.models import Case

from puzzle.plugins import BaseCaseMixin

logger = logging.getLogger(__name__)

class CaseMixin(BaseCaseMixin):
    """Class to store methods that deal with Cases in vcf plugin"""
    
    def _add_individual(self, ind_obj):
        """Add a individual to the adapter
        
            Args:
                ind_obj (puzzle.models.Individual)
        """
        logger.debug("Adding individual {0} to plugin".format(ind_obj.ind_id))
        self.individual_objs.append(ind_obj)

    def add_case(self, case_obj):
        """Add a case obj with individuals to adapter
        
            Args:
                case_obj (puzzle.models.Case)
                
        """
        for ind_obj in case_obj.individuals:
            self._add_individual(ind_obj)
        logger.debug("Adding case {0} to plugin".format(case_obj.case_id))
        self.case_objs.append(case_obj)
        if case_obj.tabix_index:
            logger.debug("Setting filters.can_filter_range to True")
            self.filters.can_filter_range = True
    
    def cases(self, pattern=None):
        """Cases found for the adapter."""

        return self.case_objs

    def case(self, case_id=None):
        """Return a Case object

            If no case_id is given return one case

            Args:
                case_id (str): A case id

            Returns:
                A Case object
        """
        if case_id:
            for case in self.case_objs:
                if case.case_id == case_id:
                    return case
        else:
            if self.cases:
                return list(self.case_objs)[0]

        return Case(case_id='unknown')
    
    def individual(self, ind_id=None):
        """Return a individual object
        
            Args:
                ind_id (str): A individual id
            
            Returns:
                individual (puzzle.models.individual)
        """
        for ind_obj in self.individual_objs:
            if ind_obj.ind_id == ind_id:
                return ind_obj
        return None
    
    def individuals(self, ind_ids=None):
        """Return information about individuals
        
            Args:
                ind_ids (list(str)): List of individual ids
            
            Returns:
                individuals (Iterable): Iterable with Individuals
        """
        if ind_ids:
            for ind_id in ind_ids:
                for ind in self.individual_objs:
                    if ind.ind_id == ind_id:
                        yield ind
        else:
            for ind in self.individual_objs:
                yield ind



from puzzle.utils import (get_most_severe_consequence, get_cytoband_coord,
                          get_omim_number, get_gene_info)


def test_get_gene_info():
    # test with HGNC symbol
    genes = list(get_gene_info(hgnc_symbols=['CHAT']))
    assert len(genes) == 1
    assert genes[0].symbol == 'CHAT'
    assert genes[0].ensembl_id == 'ENSG00000070748'

    # test with Ensembl id
    genes = list(get_gene_info(ensembl_ids=['ENSG00000156110']))
    assert len(genes) == 1
    assert genes[0].symbol == 'ADK'
    assert genes[0].ensembl_id == 'ENSG00000156110'

def test_get_most_severe_consequence():
    """Test get_most_sever_consequence(transcripts) method"""

    print("Test get_most_sever_consequence with a 'regular' transcripts list")
    transcripts = [
        {'consequence': 'transcript_ablation'}
    ]
    assert get_most_severe_consequence(transcripts) == 'transcript_ablation'

    print("Test get_most_sever_consequence with empty transcripts list")
    transcripts = []
    assert get_most_severe_consequence(transcripts) is None

    print("Test get_most_sever_consequence with 'unknown' consequence")
    transcripts = [
        {'consequence': 'unknown'}
    ]
    assert get_most_severe_consequence(transcripts) is None

    print("Test most_severe_consequence with multiple transcripts")
    transcripts = [
        {'consequence': 'inframe_deletion'},
        {'consequence': 'start_lost'},
        {'consequence': 'synonymous_variant'}
    ]
    assert get_most_severe_consequence(transcripts) == 'start_lost'

    print("Test most_severe_consequence with multiple transcripts (annotations)")
    transcripts = [
        {'consequence': 'start_lost&synonymous_variant'},
    ]
    assert get_most_severe_consequence(transcripts) == 'start_lost'

def test_get_cytoband_coord():
    """test get_cytoband_coord(chrom, pos) method"""

    print("Test get_cytoband_coord with different input formats")
    assert get_cytoband_coord('1', 3) == '1p36.33'
    assert get_cytoband_coord('chr1', 3) == '1p36.33'
    assert get_cytoband_coord('chr1', '3') == '1p36.33'

    print("Test get_cytoband_coord with non existing chromosome")
    assert get_cytoband_coord('chrMT', '3') is None

    print("Test get_cytoband_coord with non existing position")
    assert get_cytoband_coord('chrX', '155270600') is None

def test_get_omim_number():
    """Test get_omim_number(hgnc_symbol) method"""

    print("Test get_omim_number with valid hgnc_symbol")
    assert get_omim_number('IFT172') == 607386

    print("Test get_omim_number with invalid hgnc_symbol")
    assert get_omim_number('HEJ') is None

    print("Test getting phenotype")
    assert get_omim_number('MCCRP2') != get_omim_number('PLK4')
    assert get_omim_number('MCCRP2') is None
    assert get_omim_number('PLK4') == 605031



# -*- coding: utf-8 -*-


def test_gene_list(test_db):
    list_obj = test_db.gene_list('test-list')
    assert list_obj.gene_ids == ['ADK', 'KRAS', 'DIABLO']
    assert list_obj.cases[0].case_id == '636808'


def test_gene_lists(test_db):
    list_objs = test_db.gene_lists()
    assert list_objs.count() == 1


def test_add_genelist(test_db):
    list_id = 'Bruce Lee List'
    gene_ids = ['EGFR', 'PID']
    assert test_db.gene_lists().count() == 1
    new_list = test_db.add_genelist(list_id, gene_ids)
    assert isinstance(new_list.id, int)
    assert new_list.list_id == list_id
    assert test_db.gene_lists().count() == 2


def test_remove_genelist(test_db):
    list_id = 'test-list'
    assert test_db.gene_lists().count() == 1
    test_db.remove_genelist(list_id)
    assert test_db.gene_lists().count() == 0


def test_case_genelist(test_db):
    case_obj = test_db.case('636808')
    list_obj = test_db.case_genelist(case_obj)
    assert list_obj.list_id == "{}-HPO".format(case_obj.case_id)



# -*- coding: utf-8 -*-
from .base import Plugin
from .base_variant_mixin import BaseVariantMixin
from .base_case_mixin import BaseCaseMixin

from .vcf import VcfPlugin
try:
    from .gemini import GeminiPlugin
except ImportError as e:
    pass
from .sql import Store as SqlStore



from puzzle.utils import (get_file_type, get_variant_type)

def test_get_file_type_vcf(vcf_file):
    file_type = get_file_type(vcf_file)
    assert file_type == 'vcf'

def test_get_file_type_compressed_vcf(compressed_vcf_file):
    file_type = get_file_type(compressed_vcf_file)
    assert file_type == 'vcf'

def test_get_file_type_gemini(gemini_path):
    file_type = get_file_type(gemini_path)
    assert file_type == 'gemini'

def test_get_file_type_unknown():
    path = 'hello'
    file_type = get_file_type(path)
    assert file_type == 'unknown'

def test_get_variant_type_snv_vcf(vcf_file):
    variant_type = get_variant_type(variant_source=vcf_file)
    assert variant_type == 'snv'

def test_get_variant_type_sv_vcf(vcf_file_sv):
    variant_type = get_variant_type(variant_source=vcf_file_sv)
    assert variant_type == 'sv'

def test_get_variant_type_snv_gemini(gemini_path):
    variant_type = get_variant_type(variant_source=gemini_path)
    assert variant_type == 'snv'



# -*- coding: utf-8 -*-
import logging

from puzzle.models.sql import Suspect

logger = logging.getLogger(__name__)


class SuspectActions(object):
    def add_suspect(self, case_obj, variant_obj):
        """Link a suspect to a case."""
        new_suspect = Suspect(case=case_obj, variant_id=variant_obj.variant_id,
                              name=variant_obj.display_name)
        self.session.add(new_suspect)
        self.save()
        return new_suspect

    def suspect(self, suspect_id):
        """Fetch a suspect."""
        return self.query(Suspect).get(suspect_id)

    def delete_suspect(self, suspect_id):
        """De-link a suspect from a case."""
        suspect_obj = self.suspect(suspect_id)
        logger.debug("Deleting suspect {0}".format(suspect_obj.name))
        self.session.delete(suspect_obj)
        self.save()



# -*- coding: utf-8 -*-
import logging

from sqlite3 import DatabaseError

from gemini import GeminiQuery

from puzzle.plugins import Plugin
from . import (CaseMixin, VariantMixin)

logger = logging.getLogger(__name__)

class GeminiPlugin(CaseMixin, VariantMixin, Plugin):
    """This is the base class for puzzle plugins

        Args:
            db(str): Path to gemini database
            vtype: Variant type (snv or sv)

    """

    def __init__(self, vtype='snv'):
        super(GeminiPlugin, self).__init__()
        logger.debug("Setting variant type to {0}".format(vtype))
        self.variant_type = vtype
        self.case_objs = []
        self.individual_objs = []
        self.db = None

        self.filters.can_filter_gene = True
        self.filters.can_filter_frequency = True
        self.filters.can_filter_cadd = True
        self.filters.can_filter_consequence = True
        self.filters.can_filter_impact_severity = True
        self.filters.can_query_gemini = True
        self.filters.can_filter_sv_len = True
        self.filters.can_filter_range = True
    
    def test_gemini_db(self):
            """Check if self.db is a valid gemini database
                Raises sqlite3.DatabaseError if not a valid databse
            """
            gq = GeminiQuery(self.db)
            return True

    def init_app(self, app):
        """Initialize plugin via Flask."""
        pass



# -*- coding: utf-8 -*-
import os
import shutil
import pytest
import logging

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

# from puzzle.factory import create_app
from puzzle.models import (Variant, DotDict, Individual)
from puzzle.models.sql import BASE
from puzzle.models.sql import Case as SqlCase
from puzzle.plugins import VcfPlugin, SqlStore
from puzzle.utils import (get_cases, get_header)
# from puzzle.settings import TestConfig

from puzzle.log import configure_stream

root_logger = configure_stream()
logger = logging.getLogger(__name__)


@pytest.fixture
def vcf():
    db = VcfPlugin()
    return db

@pytest.fixture(scope='function')
def puzzle_dir(request, dir_path):
    """Return a puzzle dir with a database initialized"""
    db_path = os.path.join(dir_path, 'puzzle_db.sqlite3')
    logger.debug("db path is: {}".format(db_path))

    resource_dir = os.path.join(dir_path, 'resources')
    logger.debug("resource dir is: {}".format(resource_dir))

    logger.debug("Create directory: {0}".format(resource_dir))
    os.makedirs(resource_dir)
    logger.debug('Directory created')

    logger.debug('Connect to database and create tables')
    store = SqlStore(db_path)
    store.set_up(reset=False)

    # It's getting tear downed by dir_path()...

    return dir_path

@pytest.fixture(scope='function')
def gemini_path(request):
    """Return the path of a gemini database"""
    gemini_db = "tests/fixtures/HapMapFew.db"
    return gemini_db


@pytest.fixture(scope='function')
def vcf_file(request):
    "Return the path to the hapmap vcf"
    hapmap = "tests/fixtures/hapmap.vcf"
    return hapmap

@pytest.fixture(scope='function')
def vcf_file_no_ind(request):
    "Return the path to the hapmap vcf"
    hapmap = "tests/fixtures/no_ind.vcf"
    return hapmap

@pytest.fixture(scope='function')
def vcf_file_sv(request):
    "Return the path to the hapmap vcf with sv variants"
    hapmap = "tests/fixtures/hapmap.sv.vep.vcf.gz"
    return hapmap

@pytest.fixture(scope='function')
def compressed_vcf_file(request):
    "Return the path to the compressed hapmap vcf"
    hapmap = "tests/fixtures/hapmap.vcf.gz"
    return hapmap

@pytest.fixture(scope='function')
def indexed_vcf_file(request):
    "Return the path to a hapmap vcf with tabix index"
    hapmap = "tests/fixtures/hapmap_pos.vcf.gz"
    return hapmap

@pytest.fixture(scope='function')
def root_path(request):
    "Return the path to the test root dir"
    root = "tests/fixtures/"
    return root


@pytest.fixture(scope='function')
def ped_file(request):
    "Return the path to the hapmap vcf"
    hapmap = "tests/fixtures/hapmap.ped"
    return hapmap


@pytest.fixture(scope='function')
def gemini_db_path(request):
    "Return the path to the hapmap gemini db"
    hapmap = "tests/fixtures/HapMapFew.db"
    return hapmap

@pytest.fixture(scope='function')
def gemini_sv_db_path(request):
    "Return the path to the hapmap gemini db"
    hapmap = "tests/fixtures/HapMapSv.db"
    return hapmap

@pytest.fixture(scope='function')
def populated_puzzle_db(request, dir_path, case_obj):
    """Return a puzzle dir with a populated database"""
    db_path = os.path.join(dir_path, 'puzzle_db.sqlite3')
    logger.debug("db path is: {}".format(db_path))

    resource_dir = os.path.join(dir_path, 'resources')
    logger.debug("resource dir is: {}".format(resource_dir))

    logger.debug("Create directory: {0}".format(resource_dir))
    os.makedirs(resource_dir)
    logger.debug('Directory created')

    logger.debug('Connect to database and create tables')
    store = SqlStore(db_path)
    store.set_up(reset=False)

    store.add_case(case_obj, vtype='sv', mode='vcf')

    #It's getting tear downed by dir_path()...

    return dir_path


@pytest.fixture(scope='function')
def dir_path(request):
    """Return the path to a dir. Delete afterwards"""
    path_to_dir = "tests/fixtures/test_dir"

    def teardown():
        print('\n')
        logger.info("Teardown directory")
        if os.path.exists(path_to_dir):
            shutil.rmtree(path_to_dir)
    request.addfinalizer(teardown)

    return path_to_dir


@pytest.fixture
def variant():
    """Return a variant dictionary"""
    data = dict(CHROM='1', POS='100', ID='rs01', REF='A', ALT='T', QUAL='100',
                FILTER='PASS')
    variant = Variant(**data)
    return variant


@pytest.fixture(scope='session')
def session(request):
    """Create and return a session to a sqlite database"""
    engine = create_engine("sqlite:///:memory:")
    connection = engine.connect()

    session = sessionmaker()
    session.configure(bind=engine)
    BASE.metadata.create_all(engine)

    s = session()

    def teardown():
        print('\n')
        logger.info("Teardown sqlite database")
        s.close()
        connection.close()
    request.addfinalizer(teardown)

    return s

@pytest.fixture(scope='function')
def individual():
    """Return a individual object"""
    ind_obj = (Individual('1'))
    return ind_obj


@pytest.yield_fixture(scope='session')
def ped_lines():
    """Return an unformatted list of ped lines."""
    _ped_lines = [
        "636808\tADM1059A1\t0\t0\t1\t1",
        "636808\tADM1059A2\tADM1059A1\tADM1059A3\t1\t2",
        "636808\tADM1059A3\t0\t0\t2\t1"
    ]
    yield _ped_lines


@pytest.yield_fixture(scope='function')
def case_obj(ped_lines):
    """Return a test case object with individuals."""
    _case = get_cases('tests/fixtures/hapmap.vcf', case_lines=ped_lines)[0]
    yield _case

# @pytest.yield_fixture(scope='function')
# def sql_case_obj(case_obj):
#     """Return a test case for the sql model."""
#     _sql_case = SqlCase(case_id=case_obj.case_id,
#                     name=case_obj.name,
#                     variant_source=case_obj.variant_source,
#                     variant_type=case_obj.variant_type,
#                     variant_mode=case_obj.variant_type,
#                     compressed=case_obj.compressed,
#                     tabix_index=case_obj.tabix_index)
#
#     yield _sql_case

@pytest.yield_fixture(scope='function')
def gemini_case_obj(gemini_db_path):
    """Return a case object extracted from gemini database"""
    _case = get_cases(gemini_db_path, variant_mode='gemini')[0]
    yield _case

@pytest.yield_fixture(scope='function')
def gemini_sv_case_obj(gemini_sv_db_path):
    """Return a case object extracted from gemini database"""
    _case = get_cases(gemini_sv_db_path, variant_mode='gemini')[0]
    _case['variant_type'] = 'sv'
    yield _case

@pytest.yield_fixture(scope='function')
def sql_store():
    """Setup an in-memory database."""
    _test_db = SqlStore('sqlite:///:memory:')
    _test_db.set_up()
    yield _test_db
    _test_db.tear_down()


@pytest.yield_fixture(scope='function')
def test_db(sql_store, case_obj):
    """Populate database with some sample data."""
    new_case = sql_store.add_case(case_obj)
    sql_store.add_gemini_query('Chuck Norris Query', "SELECT * FROM variants")
    sql_store.add_genelist('test-list', ['ADK', 'KRAS', 'DIABLO'],
                           case_obj=new_case)
    yield sql_store
    sql_store.tear_down()
    sql_store.set_up()

@pytest.yield_fixture(scope='session')
def phenomizer_auth():
    """Fetch phenomizer auth details from ENV variables."""
    raw_auth = os.environ['PHENOMIZER_AUTH']
    auth = raw_auth.split()
    yield auth

@pytest.fixture(scope='function')
def header(vcf_file):
    """Return a header object."""
    head = get_header(vcf_file)
    return head

@pytest.fixture(scope='function')
def cyvcf_variant(request):
    "Return dictionary to mock a cyvcf variant"
    variant = DotDict()
    variant.CHROM = 'X'
    variant.POS = 84563218
    variant.REF = 'C'
    variant.ALT = ['G']
    variant.FILTER = 'PASS'
    variant.ID ='rs1'
    variant.QUAL = 360.829986572
    variant.INFO = {
            'CADD': 25,
            'CSQ': "G|missense_variant|MODERATE|POF1B|ENSG00000124429|"\
            "Transcript|ENST00000373145|protein_coding|10/16||ENST00000373145"\
            ".3:c.962G>C|ENSP00000362238.3:p.Arg321Thr|1082|962|321|R/T|aGg/"\
            "aCg|||-1|HGNC|13711|||ENSP00000362238|POF1B_HUMAN||UPI00001AE9F1"\
            "|deleterious|possibly_damaging|hmmpanther:PTHR22546|||||,G|"\
            "missense_variant|MODERATE|POF1B|ENSG00000124429|Transcript|"\
            "ENST00000262753|protein_coding|10/17||ENST00000262753.4:c.962G"\
            ">C|ENSP00000262753.4:p.Arg321Thr|1108|962|321|R/T|aGg/aCg|||-"\
            "1|HGNC|13711||CCDS14452.1|ENSP00000262753|POF1B_HUMAN||"\
            "UPI0000212116|deleterious|probably_damaging|hmmpanther:PTHR22546|||||",
        'Compounds': '643594:X_84615532_GTA_G>12',
        'Ensembl_gene_id': 'ENSG00000124429',
        'GeneticModels': '643594:XD_dn|AR_comp_dn|XR_dn',
        'ModelScore': '643594:16.0',
        'RankScore': '643594:19',
         }
    variant.start = 84563217
    variant.end = 84563218
    variant.var_type = 'snp'
    variant.sub_vartype = 'tv'
    variant.gt_types = ['C/C', 'C/G', 'C/C']
    variant.gt_types = [0, 1, 0]
    variant.gt_depths = [20,  7, 20]
    variant.gt_ref_depths = [20,  1, 20]
    variant.gt_alt_depths = [0, 6, 0]
    variant.gt_quals = [ 57.,  16.,  54.]
    variant.aaf = 0.16666666666666666
    variant.call_rate =  1.0
    variant.gt_phases = [False, False, False]
    variant.gt_phred_ll_het = [57,0,54]
    variant.gt_phred_ll_homalt = [855,16,810]
    variant.gt_phred_ll_homref = [0,154,0]
    variant.is_deletion = False
    variant.is_indel = False
    variant.is_snp = True
    variant.is_sv = False
    variant.is_transition = False
    variant.nucl_diversity = 0.3333333333333333
    variant.num_called = 3
    variant.num_het = 1
    variant.num_hom_alt = 0
    variant.num_hom_ref = 2
    variant.num_unknown = 0
    
    return variant


@pytest.fixture(scope='function')
def gemini_variant(request):
    "Return dictionary to mock a gemini variant"
    variant = {
        'chrom': 'chr1',
        'start': 36636644,
        'end': 36636645,
        'variant_id': 1,
        'ref': 'G',
        'alt': 'A',
        'qual': 360.829986572,
        'type': 'snp',
        'sub_type': 'ts',
        'gts': ['G/A', 'G/A', './.', './.', './.'],
        'gt_types': [1, 1, 2, 2, 2],
        'gt_depths': [17, 19, -1, -1, -1],
        'gt_ref_depths': [10, 13, -1, -1, -1],
        'gt_alt_depths': [7, 6, -1, -1, -1],
        'gt_quals': [99, 99, -1, -1, -1],
        'in_dbsnp': 0,
        'rs_ids': None,
        'gene': 'MAP7D1',
        'transcript': 'ENST00000530729',
        'is_exonic': 1,
        'is_coding': 1,
        'is_lof': 1,
        'exon': 2/4,
        'codon_change': 'atG/atA',
        'aa_change': 'M/I',
        'aa_length': '1/140',
        'biotype': 'protein_coding',
        'impact': 'transcript_codon_change',
        'impact_so': 'initiator_codon_variant',
        'impact_severity': 'HIGH',
        'polyphen_pred': 'benign',
        'polyphen_score': 0.304,
        'sift_pred': 'deleterious',
        'sift_score': 0.0,
        'aaf_esp_ea': None,
        'aaf_esp_aa': None,
        'aaf_esp_all': None,
        'aaf_1kg_amr': None,
        'aaf_1kg_eas': None,
        'aaf_1kg_sas': None,
        'aaf_1kg_afr': None,
        'aaf_1kg_eur': None,
        'aaf_1kg_all': None,
        'cadd_raw': 2.49,
        'cadd_scaled': 14.3,
        'aaf_exac_all': 1.647e-05,
        'aaf_adj_exac_all': 1.67162582328e-05,
        'max_aaf_all': 3.03471716436e-05,
    }
    return variant



# -*- coding: utf-8 -*-
from puzzle.plugins import VcfPlugin


def test_variants_case(case_obj):
    vcf_plugin = VcfPlugin()
    vcf_plugin.add_case(case_obj)
    #case_id is 'hapmap.vcf' since no ped file is given
    result = vcf_plugin.variants(case_id=case_obj.case_id)
    variants = result.variants
    variant = variants[0]
    assert variant['CHROM'] == 'X'
    assert int(variant['POS']) == 84563218
    assert variant['index'] == 1

    variant = variants[1]
    assert variant['CHROM'] == '2'
    assert int(variant['POS']) == 233349186
    assert variant['index'] == 2

def test_variant(case_obj):
    vcf_plugin = VcfPlugin()
    vcf_plugin.add_case(case_obj)

    variant = vcf_plugin.variant(case_obj.case_id, 'X_155239821_G_A')
    assert variant['CHROM'] == 'X'
    assert int(variant['POS']) == 155239821

    # get 10th variant
    variant = vcf_plugin.variant(case_obj.case_id, '3_124998098_C_A')
    assert variant['index'] == 10

def test_format_variants(cyvcf_variant, case_obj, header):
    vcf_plugin = VcfPlugin()
    vcf_plugin.head = header
    vcf_plugin.vep_header = header.vep_columns
    vcf_plugin.snpeff_header = header.snpeff_columns

    variant_obj = vcf_plugin._format_variants(cyvcf_variant, index=1,
                             case_obj=case_obj, add_all_info=False)

    assert variant_obj.CHROM == cyvcf_variant.CHROM
    assert variant_obj.start == cyvcf_variant.start

class TestFilters:

    def test_filters_no_filter(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {}
        result = plugin.variants(case_id, filters=filters, count=1000)
        
        assert result.nr_of_variants == 108

    def test_filters_frequency(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'frequency':'0.001'}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        for variant_obj in variants:
            assert variant_obj.max_freq <= 0.001
        
        assert nr_of_variants == 73

    def test_filters_cadd(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'cadd':'20'}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            variant_obj.cadd_score >= 20
        
        assert nr_of_variants == 50

    def test_filters_impact_severities_high(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'impact_severities':['HIGH']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'HIGH'
        
        assert nr_of_variants == 7

    def test_filters_impact_severities_medium(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'impact_severities':['MEDIUM']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'MEDIUM'
        
        assert nr_of_variants == 82

    def test_filters_impact_severities_low(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'impact_severities':['LOW']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'LOW'
        
        assert nr_of_variants == 19

    def test_filters_impact_severities_high_and_med(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'impact_severities':['HIGH', 'MEDIUM']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity in ['HIGH', 'MEDIUM']
        
        assert nr_of_variants == 89

    def test_filters_gene_ids(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'gene_ids':['POF1B']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert 'POF1B' in variant_obj.gene_symbols
        
        assert nr_of_variants == 1

    def test_filters_consequence(self, case_obj):
        plugin = VcfPlugin()
        plugin.add_case(case_obj)
        case_id = case_obj.case_id

        filters = {'consequence':['frameshift_variant']}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert 'frameshift_variant' in variant_obj.consequences
        
        assert nr_of_variants == 4

    def test_filters_range(self, case_obj, indexed_vcf_file):
        plugin = VcfPlugin()
        case_obj.variant_source = indexed_vcf_file
        case_obj.compressed = True
        case_obj.tabix_index = True
        
        plugin.add_case(case_obj)
        case_id = case_obj.case_id
        start = 1771120
        end = 1771130

        filters = {'range':{'chromosome':'1', 'start':start, 'end':end}}
        result = plugin.variants(case_id, filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.start >= start
            assert variant_obj.stop <= end
        
        assert nr_of_variants == 1



# -*- coding: utf-8 -*-
import logging

from . import (DotDict, PedigreeHumanMixin)

logger = logging.getLogger(__name__)

class Genotype(DotDict, PedigreeHumanMixin):
    """Class that holds information about a genotype call"""
    def __init__(self, sample_id, genotype, case_id=None, phenotype=None,
                ref_depth='.', alt_depth='.', genotype_quality='.', depth='.',
                supporting_evidence='0', pe_support='0', sr_support='0'):
        super(Genotype, self).__init__(sample_id=sample_id, genotype=genotype,
            case_id=case_id, phenotype=phenotype, ref_depth=ref_depth, 
            alt_depth=alt_depth, depth=depth, genotype_quality=genotype_quality,
            supporting_evidence=supporting_evidence, pe_support=pe_support, 
            sr_support=sr_support)



# -*- coding: utf-8 -*-


def test_add_resource(test_db):
    ind_obj = test_db.individual('ADM1059A2')
    name = 'test-resource'
    file_path = './tests/fixtures/gene-list.txt'
    new_resource_obj = test_db.add_resource(name, file_path, ind_obj)
    assert isinstance(new_resource_obj.id, int)

    # test getting a resource
    resource_obj = test_db.resource(new_resource_obj.id)
    assert resource_obj == new_resource_obj

    # delete resource
    test_db.delete_resource(resource_obj.id)
    assert test_db.resource(resource_obj.id) is None



# -*- coding: utf-8 -*-
from puzzle.models.sql import GeneList, CaseGenelistLink


class GeneListActions(object):
    def gene_list(self, list_id):
        """Get a gene list from the database."""
        return self.query(GeneList).filter_by(list_id=list_id).first()

    def gene_lists(self):
        """Return all gene lists from the database."""
        return self.query(GeneList)

    def add_genelist(self, list_id, gene_ids, case_obj=None):
        """Create a new gene list and optionally link to cases."""
        new_genelist = GeneList(list_id=list_id)
        new_genelist.gene_ids = gene_ids
        if case_obj:
            new_genelist.cases.append(case_obj)

        self.session.add(new_genelist)
        self.save()
        return new_genelist

    def remove_genelist(self, list_id, case_obj=None):
        """Remove a gene list and links to cases."""
        gene_list = self.gene_list(list_id)

        if case_obj:
            # remove a single link between case and gene list
            case_ids = [case_obj.id]
        else:
            # remove all links and the list itself
            case_ids = [case.id for case in gene_list.cases]
            self.session.delete(gene_list)

        case_links = self.query(CaseGenelistLink).filter(
            CaseGenelistLink.case_id.in_(case_ids),
            CaseGenelistLink.genelist_id == gene_list.id
        )
        for case_link in case_links:
            self.session.delete(case_link)

        self.save()

    def case_genelist(self, case_obj):
        """Get or create a new case specific gene list record."""
        list_id = "{}-HPO".format(case_obj.case_id)
        gene_list = self.gene_list(list_id)

        if gene_list is None:
            gene_list = GeneList(list_id=list_id)
            case_obj.gene_lists.append(gene_list)
            self.session.add(gene_list)

        return gene_list



# -*- coding: utf-8 -*-
from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

from .models import BASE


class Resource(BASE):

    """Represent an external resource."""

    __tablename__ = "resource"

    id = Column(Integer, primary_key=True)
    name = Column(String(32), nullable=False)
    path = Column(String(128), nullable=False)
    description = Column(String(128))

    ind_id = Column(Integer, ForeignKey('individual.id'))
    individual = relationship('Individual', backref=('resources'))

    def __repr__(self):
        return ("Resource(filename={this.filename})".format(this=self))



from . import (FrequenciesExtras, GenotypeExtras, TranscriptExtras, 
               ConsequenceExtras, GeneExtras)

#GeneExtras, AnnotationExtras
class VariantExtras(FrequenciesExtras, GenotypeExtras, TranscriptExtras, 
ConsequenceExtras, GeneExtras):
    """docstring for VariantExtras"""
    pass



import logging
import operator

from puzzle.models import (Compound)

logger = logging.getLogger(__name__)

class AnnotationExtras(object):
    """Mixin class to store methods that deals with parsing annotations"""
    
    def _add_compounds(self, variant_obj, info_dict):
        """Check if there are any compounds and add them to the variant
        
            The compounds that are added should be sorted on rank score
        """
        compound_list = []
        compound_entry = info_dict.get('Compounds')
        if compound_entry:
            for family_annotation in compound_entry.split(','):
                compounds = family_annotation.split(':')[-1].split('|')
                for compound in compounds:
                    splitted_compound = compound.split('>')

                    compound_score = None
                    if len(splitted_compound) > 1:
                        compound_id = splitted_compound[0]
                        compound_score = int(splitted_compound[-1])
                    
                    compound_list.append(Compound(
                        variant_id=compound_id,
                        combined_score=compound_score
                        )
                    )
        
        #Sort the compounds based on rank score
        compound_list.sort(key = operator.attrgetter('combined_score'), reverse=True)
        
        for compound in compound_list:
            variant_obj.add_compound(compound)

    def _add_cadd_score(self, variant_obj, info_dict):
        """Add the cadd score to the variant
        
            Args:
                variant_obj (puzzle.models.Variant)
                info_dict (dict): A info dictionary
        """
        cadd_score = info_dict.get('CADD')
        if cadd_score:
            logger.debug("Updating cadd_score to: {0}".format(
                cadd_score))
            variant_obj.cadd_score = float(cadd_score)
        ##TODO if cadd score is annotated with vep or snpeff,
        ## extract from transcripts
    
    def _add_genetic_models(self, variant_obj, info_dict):
        """Add the genetic models found
        
        Args:
            variant_obj (puzzle.models.Variant)
            info_dict (dict): A info dictionary
        
        """
        genetic_models_entry = info_dict.get('GeneticModels')
        if genetic_models_entry:
            genetic_models = []
            for family_annotation in genetic_models_entry.split(','):
                for genetic_model in family_annotation.split(':')[-1].split('|'):
                    genetic_models.append(genetic_model)
            logger.debug("Updating genetic models to: {0}".format(
                ', '.join(genetic_models)))
                
            variant_obj.genetic_models = genetic_models
    
    def _add_rank_score(self, variant_obj, info_dict):
        """Add the rank score if found
        
        Args:
            variant_obj (puzzle.models.Variant)
            info_dict (dict): A info dictionary
        
        """
        rank_score_entry = info_dict.get('RankScore')
        if rank_score_entry:
            for family_annotation in rank_score_entry.split(','):
                rank_score = family_annotation.split(':')[-1]
            logger.debug("Updating rank_score to: {0}".format(
                rank_score))
            variant_obj.rank_score = float(rank_score)
    


# -*- coding: utf-8 -*-
from datetime import datetime

from sqlalchemy import Column, types, ForeignKey
from sqlalchemy.orm import relationship

from .models import BASE


class Comment(BASE):

    """Represent a comment on case or variant level."""

    __tablename__ = "comment"

    id = Column(types.Integer, primary_key=True)
    text = Column(types.Text, nullable=False)
    username = Column(types.String(32), default='Anonymous')
    created_at = Column(types.DateTime, default=datetime.now)

    case_id = Column(types.Integer, ForeignKey('case.id'), nullable=False)
    case = relationship('Case', backref=('comments'))
    # md5 sum of chrom, pos, ref, alt
    variant_id = Column(types.String(32))

    def __repr__(self):
        return ("Comment(name={this.username}, text={this.text}"
                .format(this=self))



# -*- coding: utf-8 -*-
from .phenotype import PhenotypeActions
from .gemini import GeminiActions
from .genelist import GeneListActions
from .resource import ResourceActions
from .comment import CommentActions
from .suspect import SuspectActions

class ActionsMixin(PhenotypeActions, GeminiActions, GeneListActions,
                   ResourceActions, CommentActions, SuspectActions):
    pass



# -*- coding: utf-8 -*-
import itertools
import logging

from puzzle.plugins import BaseVariantMixin

logger = logging.getLogger(__name__)


class VariantMixin(BaseVariantMixin):
    def variants(self, case_id, skip=0, count=1000, filters=None):
        """Fetch variants for a case."""
        filters = filters or {}
        logger.debug("Fetching case with case_id: {0}".format(case_id))
        case_obj = self.case(case_id)
        plugin, case_id = self.select_plugin(case_obj)
        self.filters = plugin.filters

        gene_lists = (self.gene_list(list_id) for list_id
                      in filters.get('gene_lists', []))
        nested_geneids = (gene_list.gene_ids for gene_list in gene_lists)
        gene_ids = set(itertools.chain.from_iterable(nested_geneids))

        if filters.get('gene_ids'):
            filters['gene_ids'].extend(gene_ids)
        else:
            filters['gene_ids'] = gene_ids
        variants = plugin.variants(case_id, skip, count, filters)
        return variants

    def variant(self, case_id, variant_id):
        """Fetch a single variant from variant source."""
        case_obj = self.case(case_id)
        plugin, case_id = self.select_plugin(case_obj)
        variant = plugin.variant(case_id, variant_id)
        return variant



# -*- coding: utf-8 -*-
from .dotdict import DotDict
from .mixins import PedigreeHumanMixin
from .individual import Individual
from .case import Case
from .transcript import Transcript
from .gene import Gene
from .compound import Compound
from .genotype import Genotype
from .variant import (Variant, Genotype, Gene,
Compound)



# -*- coding: utf-8 -*-
import logging

from puzzle.models.sql import Comment

logger = logging.getLogger(__name__)


class CommentActions(object):
    def comments(self, case_id=None, variant_id=None, username=None):
        """Return comments for a case or variant.

        Args:
            case_id (str): id for a related case
            variant_id (Optional[str]): id for a related variant
        """
        logger.debug("Looking for comments")
        comment_objs = self.query(Comment)

        if case_id:
            comment_objs = comment_objs.filter_by(case_id=case_id)

        if variant_id:
            comment_objs = comment_objs.filter_by(variant_id=variant_id)
        elif case_id:
            comment_objs = comment_objs.filter_by(variant_id=None)

        return comment_objs

    def comment(self, comment_id):
        """Fetch a comment.

        Args:
            comment_id (int): record id from the database
        """
        return self.query(Comment).get(comment_id)

    def add_comment(self, case_obj, text, variant_id=None, username=None):
        """Add a comment to a variant or a case"""

        comment = Comment(
            text=text,
            username=username or 'Anonymous',
            case=case_obj,
            # md5 sum of chrom, pos, ref, alt
            variant_id=variant_id
        )
        self.session.add(comment)
        self.save()
        return comment

    def delete_comment(self, comment_id):
        """Delete a comment"""

        comment_obj = self.query(Comment).get(comment_id)
        if comment_obj:
            logger.debug("Deleting comment {0}".format(comment_obj.id))

        self.session.delete(comment_obj)
        self.save()

        return comment_obj

    def update_synopsis(self, case_obj, text):
        """Update the synopsis for a case."""
        case_obj.synopsis = text
        self.save()



# -*- coding: utf-8 -*-


def test_add_case(sql_store, case_obj):
    sql_store.add_case(case_obj)
    case = sql_store.case(case_obj['case_id'])
    assert case and case.case_id == case_obj['case_id']


def test_delete_case(test_db):
    case_obj = test_db.case('636808')
    test_db.delete_case(case_obj)

    empty_case = test_db.case('636808')
    assert empty_case is None


def test_delete_individual(test_db):
    # GIVEN: a single case in the database
    case_obj = test_db.cases().all()[0]

    # WHEN: the case is delete
    test_db.delete_case(case_obj)

    # THEN: no cases are left in the database
    assert test_db.cases().count() == 0


def test_case(test_db):
    # GIVEN: a case "636808" in the database
    case_obj = test_db.case('636808')

    # WHEN: -

    # THEN: the returned case has case_id: "636808"
    assert case_obj.case_id == '636808'


def test_individual(test_db):
    # GIVEN: an individual "ADM1059A1" in the database
    ind_obj = test_db.individual('ADM1059A1')

    # WHEN: -

    # THEN: the returned case has ind_id: "ADM1059A1"
    assert ind_obj.ind_id == 'ADM1059A1'
    assert ind_obj.cases[0].case_id == '636808'


def test_cases(test_db):
    # GIVEN: a single case in the database
    cases_query = test_db.cases()

    # THEN: fetching all cases should return one object as a list
    assert cases_query.count() == 1


def test_get_individuals(test_db):
    # GIVEN: a case with three individuals in the database
    individuals = test_db.individuals()

    # THEN: fetching individuals returns three records
    assert individuals.count() == 3

    # WHEN: passing in two ind ids
    ind_ids = [ind.ind_id for ind in individuals]
    sub_inds = test_db.individuals(ind_ids=ind_ids[:2])

    # THEN: ... it returns two records matching the ids
    assert sub_inds.count() == 2
    assert [ind.ind_id for ind in sub_inds] == ind_ids[:2]



# -*- coding: utf-8 -*-
import os
import tempfile
import webbrowser

import click
import logging

from path import path

from . import (base, family_file, family_type, version, root, phenomizer)

from puzzle.plugins import SqlStore, VcfPlugin
try:
    from puzzle.plugins import GeminiPlugin
    GEMINI = True
except ImportError:
    GEMINI = False

from sqlite3 import DatabaseError

from puzzle.server import create_app
from puzzle.server.settings import BaseConfig
from puzzle.utils import (get_file_type, get_variant_type, get_cases)

logger = logging.getLogger(__name__)


@base.command()
@click.argument('variant-source', type=click.Path(exists=True), required=False)
@click.option('--host', default='0.0.0.0', show_default=True)
@click.option('--port', default=5000, show_default=True)
@click.option('--debug', is_flag=True)
@click.option('-p', '--pattern', default='*', show_default=True)
@click.option('--no-browser', is_flag=True, help='Prevent auto-opening browser')
@phenomizer
@family_file
@family_type
@version
@root
@click.pass_context
def view(ctx, host, port, debug, pattern, family_file, family_type,
         variant_source, root, no_browser, phenomizer):
    """Visualize DNA variant resources.

    1. Look for variant source(s) to visualize and inst. the right plugin
    """
    main_loop = (not debug) or (os.environ.get('WERKZEUG_RUN_MAIN') == 'true')
    root = root or ctx.obj.get('root') or os.path.expanduser("~/.puzzle")
    phenomizer_auth = phenomizer or ctx.obj.get('phenomizer_auth')
    BaseConfig.PHENOMIZER_AUTH = True if ctx.obj.get('phenomizer_auth') else False
    BaseConfig.STORE_ENABLED = True

    if variant_source is None:
        logger.info("Root directory is: {}".format(root))

        db_path = os.path.join(root, 'puzzle_db.sqlite3')
        logger.info("db path is: {}".format(db_path))
        if not os.path.exists(db_path):
            logger.warn("database not initialized, run 'puzzle init'")
            ctx.abort()

        if os.path.isfile(root):
            logger.error("'root' can't be a file")
            ctx.abort()

        store = SqlStore(db_path, phenomizer_auth=phenomizer_auth)
        for case_obj in store.cases():
            if case_obj.variant_mode == 'gemini':
                if not GEMINI:
                    logger.error("Need to have gemini instaled to view gemini database")
                    ctx.abort()

    else:
        logger.info("Using in memory database")
        tmpdir = tempfile.mkdtemp()
        tmpdb = os.path.join(tmpdir, 'puzzle.sqlite3')
        logger.info("building database: {}".format(tmpdb))
        store = SqlStore("sqlite:///{}".format(tmpdb),
                         phenomizer_auth=phenomizer_auth)
        if main_loop:
            store.set_up()
            cases = []
            if os.path.isfile(variant_source):
                file_type = get_file_type(variant_source)
                #Test if gemini is installed
                if file_type == 'unknown':
                    logger.error("File has to be vcf or gemini db")
                    ctx.abort()
                elif file_type == 'gemini':
                    #Check if gemini is installed
                    if not GEMINI:
                        logger.error("Need to have gemini installed to use gemini plugin")
                        ctx.abort()
                variant_type = get_variant_type(variant_source)
                cases = get_cases(
                    variant_source=variant_source,
                    case_lines=family_file,
                    case_type=family_type,
                    variant_type=variant_type,
                    variant_mode=file_type
                )
            else:
                for file in path(variant_source).walkfiles(pattern):
                    file_type = get_file_type(file)
                    if file_type != 'unknown':
                        variant_type = get_variant_type(file)
                        #Test if gemini is installed
                        if file_type == 'gemini':
                            if not GEMINI:
                                logger.error("Need to have gemini installed to use gemini plugin")
                                ctx.abort()

                        for case in get_cases(
                            variant_source=file,
                            case_type=family_type,
                            variant_type=variant_type,
                            variant_mode=file_type):

                            cases.append(case)

            for case_obj in cases:
                if store.case(case_obj.case_id) is not None:
                    logger.warn("{} already exists in the database"
                                .format(case_obj.case_id))
                    continue

                # extract case information
                logger.debug("adding case: {}".format(case_obj.case_id))
                store.add_case(case_obj, vtype=case_obj.variant_type, mode=case_obj.variant_mode)

    logger.debug("Plugin setup was succesfull")
    BaseConfig.PUZZLE_BACKEND = store
    BaseConfig.UPLOAD_DIR = os.path.join(root, 'resources')

    app = create_app(config_obj=BaseConfig)

    if no_browser is False:
        webbrowser.open_new_tab("http://{}:{}".format(host, port))

    app.run(host=host, port=port, debug=debug)



from puzzle.plugins import VcfPlugin

def test_add_case(case_obj):
    adapter=VcfPlugin()
    adapter.add_case(case_obj)
    assert len(adapter.individual_objs) == 3
    case_obj = adapter.case_objs[0]
    assert case_obj.name == "636808"

def test__add_individual(individual):
    adapter=VcfPlugin()
    adapter._add_individual(individual)
    assert len(adapter.individual_objs) == 1

def test_cases(case_obj):
    adapter=VcfPlugin()
    adapter.add_case(case_obj)
    case_id = "636808"
    case_obj = adapter.cases()[0]
    assert case_obj.name == "636808"

def test_case(case_obj):
    adapter=VcfPlugin()
    adapter.add_case(case_obj)
    case_id = "636808"
    case_obj = adapter.case(case_id)
    assert case_obj.case_id == case_id

def test_individual(case_obj):
    adapter=VcfPlugin()
    adapter.add_case(case_obj)
    
    ind_id = "ADM1059A1"
    ind_obj = adapter.individual(ind_id)
    assert ind_obj.ind_id == ind_id

def test_individuals(case_obj):
    adapter=VcfPlugin()
    adapter.add_case(case_obj)
    
    individuals = [ind for ind in adapter.individuals()]
    assert len(individuals) == 3



import logging

from vcftoolbox import (HeaderParser, get_vcf_handle)

logger = logging.getLogger(__name__)

def get_csq(description):
    """Get the csq columns
    
        Args:
            description (str): The vcf header description string
        
        Returns:
            csq_cols (list): The CSQ colums
    """
    csq_description = description.split(':')[1][1:-1]
    csq_cols = csq_description.split('|')
    return csq_cols

def get_header(vcf_file_path):
    """Parse the header and return a header object

        Args:
            vcf_file_path(str): Path to vcf

        Returns:
            head: A HeaderParser object
    """
    logger.info("Parsing header of file {0}".format(vcf_file_path))
    head = HeaderParser()
    handle = get_vcf_handle(infile=vcf_file_path)
    # Parse the header
    for line in handle:
        line = line.rstrip()
        if line.startswith('#'):
            if line.startswith('##'):
                head.parse_meta_data(line)
            else:
                head.parse_header_line(line)
        else:
            break

    handle.close()

    return head



import os

import logging
import click

from . import (base, root, family_file, family_type)

from puzzle.plugins import SqlStore

try:
    from puzzle.plugins import GeminiPlugin
    GEMINI = True
except ImportError:
    GEMINI=False

from puzzle.utils import (get_file_type, get_variant_type, get_cases)

from sqlite3 import DatabaseError

logger = logging.getLogger(__name__)

@base.command()
@click.argument('variant-source',
    type=click.Path(exists=True),
)
@family_file
@family_type
@root
@click.pass_context
def load(ctx, variant_source, family_file, family_type, root):
    """
    Load a variant source into the database.

    If no database was found run puzzle init first.

    1. VCF: If a vcf file is used it can be loaded with a ped file
    2. GEMINI: Ped information will be retreived from the gemini db
    """
    root = root or ctx.obj.get('root') or os.path.expanduser("~/.puzzle")

    if os.path.isfile(root):
        logger.error("'root' can't be a file")
        ctx.abort()

    logger.info("Root directory is: {}".format(root))

    db_path = os.path.join(root, 'puzzle_db.sqlite3')
    logger.info("db path is: {}".format(db_path))
    
    if not os.path.exists(db_path):
        logger.warn("database not initialized, run 'puzzle init'")
        ctx.abort()


    if not os.path.isfile(variant_source):
        logger.error("Variant source has to be a file")
        ctx.abort()
    
    mode = get_file_type(variant_source)
    if mode == 'unknown':
        logger.error("Unknown file type")
        ctx.abort()
    #Test if gemini is installed
    elif mode == 'gemini':
        logger.debug("Initialzing GEMINI plugin")
        if not GEMINI:
            logger.error("Need to have gemini installed to use gemini plugin")
            ctx.abort()

    logger.debug('Set puzzle backend to {0}'.format(mode))
    
    variant_type = get_variant_type(variant_source)
    logger.debug('Set variant type to {0}'.format(variant_type))
    
    cases = get_cases(
        variant_source=variant_source,
        case_lines=family_file, 
        case_type=family_type, 
        variant_type=variant_type, 
        variant_mode=mode
    )
    
    if len(cases) == 0:
        logger.warning("No cases found")
        ctx.abort()

    logger.info("Initializing sqlite plugin")
    store = SqlStore(db_path)

    for case_obj in cases:
        if store.case(case_obj.case_id) is not None:
            logger.warn("{} already exists in the database"
                        .format(case_obj.case_id))
            continue
        # extract case information
        logger.debug("adding case: {} to puzzle db".format(case_obj.case_id))
        store.add_case(case_obj, vtype=variant_type, mode=mode)



import itertools
import logging
from .constants import (SEVERITY_DICT, HGNC_TO_OMIM, CYTOBANDS)

from phizz.utils import (query_gene, query_gene_symbol)

from puzzle.models import Gene

logger = logging.getLogger(__name__)

def get_variant_id(variant):
    """Get a variant id from a cyvcf variant
    
        Build a variant id from chrom, pos, ref, alt
        Args:
            variant (cyvcf2.Variant)
        
        Returns:
            variant_id (str): A variant id
    """
    pass

def get_gene_symbols(chrom, start, stop):
    """Get the gene symbols that a interval overlaps"""
    gene_symbols = query_gene_symbol(chrom, start, stop)
    logger.debug("Found gene symbols: {0}".format(', '.join(gene_symbols)))
    return gene_symbols

def get_gene_info(ensembl_ids=None, hgnc_symbols=None):
    """Return the genes info based on the transcripts found

    Args:
        ensembl_ids (Optional[list]): list of Ensembl gene ids
        hgnc_symbols (Optional[list]): list of HGNC gene symbols

    Returns:
        iterable: an iterable with `Gene` objects
    """
    uniq_ensembl_ids = set(ensembl_id for ensembl_id in (ensembl_ids or []))
    uniq_hgnc_symbols = set(hgnc_symbol for hgnc_symbol in (hgnc_symbols or []))
    genes = []
    gene_data = []
    
    if uniq_ensembl_ids:
        for ensembl_id in uniq_ensembl_ids:
            for res in query_gene(ensembl_id=ensembl_id):
                gene_data.append(res) 

    elif uniq_hgnc_symbols:
        for hgnc_symbol in uniq_hgnc_symbols:
            query_res = query_gene(hgnc_symbol=hgnc_symbol)
            if query_res:
                for res in query_res:
                    gene_data.append(res)
            else:
                # If no result we add just the symbol
                gene_data.append({
                    'hgnc_symbol': hgnc_symbol,
                    'hgnc_id': None,
                    'ensembl_id': None,
                    'description': None,
                    'chrom': 'unknown',
                    'start': 0,
                    'stop': 0,
                    'hi_score': None,
                    'constraint_score': None,
                })
    for gene in gene_data:
        genes.append(Gene(
            symbol=gene ['hgnc_symbol'],
            hgnc_id=gene['hgnc_id'],
            ensembl_id=gene['ensembl_id'],
            description=gene['description'],
            chrom=gene['chrom'],
            start=gene['start'],
            stop=gene['stop'],
            location=get_cytoband_coord(gene['chrom'], gene['start']),
            hi_score=gene['hi_score'],
            constraint_score=gene['constraint_score'],
            omim_number=get_omim_number(gene['hgnc_symbol'])
            ))

    return genes

def get_most_severe_consequence(transcripts):
    """Get the most severe consequence

        Go through all transcripts and get the most severe consequence

        Args:
            transcripts (list): A list of transcripts to evaluate

        Returns:
            most_severe_consequence (str): The most severe consequence
    """
    most_severe_consequence = None
    most_severe_score = None

    for transcript in transcripts:
        for consequence in transcript['consequence'].split('&'):
            logger.debug("Checking severity score for consequence: {0}".format(
                consequence
            ))
            severity_score = SEVERITY_DICT.get(
                consequence
            )
            logger.debug("Severity score found: {0}".format(
                severity_score
            ))
            if severity_score != None:
                if most_severe_score:
                    if severity_score < most_severe_score:
                        most_severe_consequence = consequence
                        most_severe_score = severity_score
                else:
                    most_severe_consequence = consequence
                    most_severe_score = severity_score

    return most_severe_consequence

def get_cytoband_coord(chrom, pos):
    """Get the cytoband coordinate for a position

        Args:
            chrom(str): A chromosome
            pos(int): The position

        Returns:
            cytoband
    """
    chrom = chrom.strip('chr')
    pos = int(pos)
    result = None
    logger.debug("Finding Cytoband for chrom:{0} pos:{1}".format(chrom, pos))
    if chrom in CYTOBANDS:
        for interval in CYTOBANDS[chrom][pos]:
            result = "{0}{1}".format(chrom, interval.data)

    return result

def get_omim_number(hgnc_symbol):
    """Get the omim number for a hgnc symbol

        Args:
            hgnc_symbol (str): A hgnc symbol

        Returns:
            omim_number (int): The omim number
    """

    omim_number = HGNC_TO_OMIM.get(hgnc_symbol,{}).get('mim_nr', None)

    return omim_number



# -*- coding: utf-8 -*-
from puzzle.models.sql import Comment

def test_add_comment_case(test_db):
    text = 'test-comment'
    case_obj = test_db.cases().first()
    comment_obj = test_db.add_comment(case_obj, text)
    assert isinstance(comment_obj, Comment)
    assert comment_obj.text == text

def test_add_comment_variant(test_db, variant):
    text = 'variant-comment'
    case_obj = test_db.cases().first()
    comment_obj = test_db.add_comment(case_obj, text, variant.md5)

    assert isinstance(comment_obj, Comment)
    assert comment_obj.text == text
    assert comment_obj.variant_id == variant.md5

def test_get_comments_variant(test_db, variant):
    text = 'variant-comment'
    case_obj = test_db.cases().first()
    test_db.add_comment(case_obj, text, variant.md5)

    query = test_db.comments(variant_id=variant.md5)
    for comment in query:
        assert comment.case_id == case_obj.id
        assert comment.variant_id == variant.md5

    assert query.count() == 1
    comment_obj = query.first()

    assert isinstance(comment_obj, Comment)
    assert comment_obj.text == text

def test_get_comments(test_db):
    text = 'test-comment'
    case_obj = test_db.cases().first()
    test_db.add_comment(case_obj, text)
    comments = []
    for comment in test_db.comments(case_obj.id):
        assert comment.case_id == case_obj.id
        comments.append(comment)
    assert len(comments) == 1
    comment_obj = comments[0]
    assert comment_obj.text == text


def test_get_comment(test_db):
    text = 'test-comment'
    case_obj = test_db.cases().first()
    comment_obj = test_db.add_comment(case_obj, text)
    new_comment_obj = test_db.comment(comment_obj.id)
    assert new_comment_obj.text == text

def test_delete_comment(test_db):
    text = 'test-comment'
    case_obj = test_db.cases().first()
    comment_obj = test_db.add_comment(case_obj, text)

    test_db.delete_comment(comment_obj.id)

    comments = []
    for comment in test_db.comments(case_obj.id):
        comments.append(comment)
    assert len(comments) == 0


def test_update_synopsis(test_db):
    text = '# header'
    case_obj = test_db.cases().first()
    test_db.update_synopsis(case_obj, text)

    updated_case_obj = test_db.cases().first()
    assert updated_case_obj.synopsis == text



import logging

from puzzle.utils.constants import (SO_TERMS, IMPACT_SEVERITIES, SEVERITY_DICT)

logger = logging.getLogger(__name__)

class ConsequenceExtras(object):
    """Methods to handle consequences"""
    
    def _add_consequences(self, variant_obj, raw_variant_line):
        """Add the consequences found for a variant

            Args:
                variant_obj (puzzle.models.Variant)
                raw_variant_line (str): A raw vcf variant line
        """
        consequences = []
        for consequence in SO_TERMS:
            if consequence in raw_variant_line:
                consequences.append(consequence)
        
        variant_obj.consequences = consequences
    
    def _add_most_severe_consequence(self, variant_obj):
        """Add the most severe consequence
        
        Args:
            variant_obj (puzzle.models.Variant)
            
        """
        most_severe_consequence = None
        most_severe_score = None

        for consequence in variant_obj.consequences:
            logger.debug("Checking severity score for consequence: {0}".format(
                consequence))
            
            severity_score = SEVERITY_DICT.get(consequence)
            
            if severity_score != None:
                if most_severe_score:
                    if severity_score < most_severe_score:
                        most_severe_consequence = consequence
                        most_severe_score = severity_score
                else:
                    most_severe_consequence = consequence
                    most_severe_score = severity_score

        variant_obj.most_severe_consequence = most_severe_consequence

    def _add_impact_severity(self, variant_obj):
        """Add the impact severity for the most severe consequence
        
            Args:
                variant_obj (puzzle.models.Variant)
        
        """
        if variant_obj.most_severe_consequence:
            variant_obj.impact_severity = IMPACT_SEVERITIES.get(
                variant_obj.most_severe_consequence
            )



import os
import logging

from puzzle.cli import base as cli
from click.testing import CliRunner


class TestBaseCommand:
    """Test the cli base command"""

    def test_base_command(self):
        logger = logging.getLogger("test_base_command")
        runner = CliRunner()
        logger.debug("Test to run base command without arguments")
        result = runner.invoke(cli, [])

        assert result.exit_code == 0

class TestInitCommand:
    """Test the cli init command"""

    def test_init_command(self, dir_path):
        logger = logging.getLogger("test_init_command")
        runner = CliRunner()
        logger.debug("Test initalize a puzzle repo")
        result = runner.invoke(cli, ['init','--root', dir_path])

        assert os.path.exists(dir_path)

    def test_init_command_file(self):
        logger = logging.getLogger("test_init_command_file")
        file_path = "tests/fixtures/hapmap.ped"
        runner = CliRunner()
        logger.debug("Test initalize a puzzle repo")
        result = runner.invoke(cli, ['init','--root', file_path])

        #Can not initialize with file...
        assert result.exit_code == 1

    def test_init_command_existing_dir(self, puzzle_dir):
        logger = logging.getLogger("test_init_command_existing_dir")
        runner = CliRunner()
        logger.debug("Test initalize a existing puzzle repo")
        result = runner.invoke(cli, ['init','--root', puzzle_dir])

        #Should exit since dir already exists
        assert result.exit_code == 1


class TestLoadCommand:
    """Test the cli load command"""

    def test_load_command_vcf(self, puzzle_dir, vcf_file):
        """Test to load a vcf without any ped file"""

        logger = logging.getLogger("test_load_command_vcf")
        runner = CliRunner()
        logger.debug("Test load a vcf")
        result = runner.invoke(cli, ['load','--root', puzzle_dir, vcf_file])

        assert result.exit_code == 0

    def test_load_command_vcf_with_ped(self, puzzle_dir, vcf_file, ped_file):
        """Test to load a vcf with ped file"""

        logger = logging.getLogger("test_load_command_vcf_with_ped")
        runner = CliRunner()
        logger.debug("Test load a vcf with a ped file")
        result = runner.invoke(cli, ['load','--root', puzzle_dir, '-f', ped_file, vcf_file])

        assert result.exit_code == 0

    def test_load_command_gemini(self, puzzle_dir, gemini_db_path):
        """Test to load a gemini db"""

        logger = logging.getLogger("test_load_command_gemini")
        runner = CliRunner()
        logger.debug("Test load a gemini db")
        result = runner.invoke(cli, ['load','--root', puzzle_dir, gemini_db_path])

        assert result.exit_code == 0

    def test_load_command_no_db(self, dir_path, vcf_file):
        """Test to load a gemini db"""

        logger = logging.getLogger("test_load_command_no_db")
        runner = CliRunner()
        logger.debug("Test load a vcf when no database")
        result = runner.invoke(cli, ['load','--root', dir_path, vcf_file])
        
        db_path = os.path.join(dir_path, 'puzzle_db.sqlite3')
        
        #Should exit since db does not exist
        assert result.exit_code == 1

class TestCasesCommand:
    """Test the cli cases command"""

    def test_cases_command(self, populated_puzzle_db):
        """Check if cases command work"""

        logger = logging.getLogger("test_cases_command")
        runner = CliRunner()
        logger.debug("Test if cases command work")
        result = runner.invoke(cli, ['cases','--root', populated_puzzle_db])

        assert result.exit_code == 0
        assert 'hapmap.vcf' in result.output

    def test_cases_command_file(self, vcf_file):
        """Check if cases command work"""

        logger = logging.getLogger("test_cases_command_file")
        runner = CliRunner()
        logger.debug("Test if cases command work with a file")
        result = runner.invoke(cli, ['cases','--root', vcf_file])

        assert result.exit_code == 1

    def test_cases_command_no_db(self, dir_path):
        """Check if cases command work"""

        logger = logging.getLogger("test_cases_no_db")
        runner = CliRunner()
        logger.debug("Test if cases command work without a puzzle db")
        result = runner.invoke(cli, ['cases','--root', dir_path])

        assert result.exit_code == 1

class TestIndividualsCommand:
    """Test the cli individuals command"""

    def test_individuals_command(self, populated_puzzle_db):
        """Check if cases command work"""

        logger = logging.getLogger("test_individuals_command")
        runner = CliRunner()
        logger.debug("Test if individuals command work")
        result = runner.invoke(cli, ['individuals','--root', populated_puzzle_db])

        assert result.exit_code == 0
        assert "ADM1059A1" in result.output

    def test_individuals_command_file(self, vcf_file):
        """Check if cases command work"""

        logger = logging.getLogger("test_individuals_command_file")
        runner = CliRunner()
        logger.debug("Test if individuals command work with a file")
        result = runner.invoke(cli, ['individuals','--root', vcf_file])

        assert result.exit_code == 1

    def test_individuals_command_no_db(self, dir_path):
        """Check if cases command work"""

        logger = logging.getLogger("test_individuals_no_db")
        runner = CliRunner()
        logger.debug("Test if individuals command work without a puzzle db")
        result = runner.invoke(cli, ['individuals','--root', dir_path])

        assert result.exit_code == 1

class TestDeleteCommand:
    """Test the cli delete command"""

    def test_delete_individual(self, populated_puzzle_db):
        """Check if delete individuals command work"""

        logger = logging.getLogger("test_delete_individual")
        runner = CliRunner()
        logger.debug("Test if delete individuals command work")
        result = runner.invoke(cli, ['delete','--root', populated_puzzle_db,
                                     '-i', 'ADM1059A1'])

        assert result.exit_code == 0

    def test_delete_case(self, populated_puzzle_db):
        """Check if delete individuals command work"""

        logger = logging.getLogger("test_delete_case")
        runner = CliRunner()
        logger.debug("Test if delete case command work")
        result = runner.invoke(cli, ['delete','--root', populated_puzzle_db,
                                     '-f', '636808'])

        assert result.exit_code == 0

    def test_delete_command_file(self, vcf_file):

        runner = CliRunner()
        result = runner.invoke(cli, ['delete','--root', vcf_file,
                                     '-f', '636808'])

        assert result.exit_code == 1

    def test_delete_command_no_input(self, populated_puzzle_db):

        runner = CliRunner()
        result = runner.invoke(cli, ['delete','--root', populated_puzzle_db])

        assert result.exit_code == 1

    def test_delete_command_non_existing_case(self, populated_puzzle_db):

        runner = CliRunner()
        result = runner.invoke(cli, ['delete','--root', populated_puzzle_db,
                                     '-f', 'hello'])

        assert result.exit_code == 1

    def test_delete_command_non_existing_individual(self, populated_puzzle_db):

        runner = CliRunner()
        result = runner.invoke(cli, ['delete','--root', populated_puzzle_db,
                                     '-i', 'hello'])

        assert result.exit_code == 1




from puzzle.utils import get_csq

def test_get_csq():
    description = '"Consequence annotations from Ensembl VEP. Format: Allele|Consequence"'
    csq_cols = get_csq(description)
    assert csq_cols == ['Allele', 'Consequence']


# -*- coding: utf-8 -*-
from sqlalchemy import Column, ForeignKey, Integer, String, UniqueConstraint
from sqlalchemy.orm import relationship

from .models import BASE


class PhenotypeTerm(BASE):

    """Represent a HPO phenotype term."""

    __tablename__ = "phenotype_term"
    __table_args__ = (UniqueConstraint('ind_id', 'phenotype_id',
                                       name='_ind_phenotype_uc'),)

    id = Column(Integer, primary_key=True)
    phenotype_id = Column(String(32), nullable=False)
    description = Column(String(128))

    ind_id = Column(Integer, ForeignKey('individual.id'))
    individual = relationship('Individual', backref=('phenotypes'))

    @property
    def hpo_link(self):
        """Return a HPO link."""
        return ("http://compbio.charite.de/hpoweb/showterm?id={}"
                .format(self.phenotype_id))

    @property
    def omim_link(self):
        """Return a OMIM phenotype link."""
        return "http://www.omim.org/entry/{}".format(self.phenotype_id)

    def __repr__(self):
        return ("PhenotypeTerm(phenotype_id={this.phenotype_id})"
                .format(this=self))



##TODO mock a gemini db
import pytest
from puzzle.plugins import VcfPlugin

def test_setup_no_file():
    """Test to initialize a gemini database without any file"""

    adapter = VcfPlugin()
    
    assert adapter.variant_type == 'snv'
    
    assert adapter.filters.can_filter_gene == True
    assert adapter.filters.can_filter_inheritance == True

def test_setup_variant_type():
    """docstring for test_setup_variant_type"""
    adapter = VcfPlugin('sv')
    
    assert adapter.variant_type == 'sv'



# -*- coding: utf-8 -*-
import query_phenomizer


def hpo_genes(phenotype_ids, username, password):
    """Return list of HGNC symbols matching HPO phenotype ids.

    Args:
        phenotype_ids (list): list of phenotype ids
        username (str): username to connect to phenomizer
        password (str): password to connect to phenomizer

    Returns:
        query_result: a list of dictionaries on the form
        {
            'p_value': float,
            'gene_id': str,
            'omim_id': int,
            'orphanet_id': int,
            'decipher_id': int,
            'any_id': int,
            'mode_of_inheritance': str,
            'description': str,
            'raw_line': str
        }
    """
    if phenotype_ids:
        try:
            results = query_phenomizer.query(username, password, phenotype_ids)
            return [result for result in results
                    if result['p_value'] is not None]
        except SystemExit, RuntimeError:
            pass
    return None



# -*- coding: utf-8 -*-
"""
puzzle
~~~~~~~~~~~~~~~~~~~

A new secret project :P

:copyright: (c) 2015 by Robin Andeer
:licence: MIT, see LICENCE for more details
"""
import logging
import puzzle
# Generate your own AsciiArt at:
# patorjk.com/software/taag/#f=Calvin%20S&t=Puzzle
__banner__ = r"""
╔═╗┬ ┬┌─┐┌─┐┬  ┌─┐
╠═╝│ │┌─┘┌─┘│  ├┤   by Robin Andeer
╩  └─┘└─┘└─┘┴─┘└─┘  & Måns Magnusson
"""

__title__ = 'puzzle'
__summary__ = 'A portable visualizer for DNA variations'
__uri__ = 'https://github.com/robinandeer/puzzle'

__version__ = '0.0.1'

__author__ = ['Robin Andeer', u'Måns Magnusson']
__email__ = 'robin.andeer@gmail.com'

__license__ = 'MIT'
__copyright__ = 'Copyright 2015 Robin Andeer'

# the user should dictate what happens when a logging event occurs
logging.getLogger(__name__).addHandler(logging.NullHandler())
resource_package = puzzle.__name__



import os
from puzzle.utils.ped import (get_individuals, get_cases)

class TestGetIndividuals:
    
    def test_get_individuals_from_vcf(self, vcf_file):
        individuals = get_individuals(variant_source=vcf_file)
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_individuals_from_compressed_vcf(self, compressed_vcf_file):
        individuals = get_individuals(variant_source=compressed_vcf_file)
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])
    
    def test_get_individuals_case_lines(self, vcf_file, ped_lines):
        individuals = get_individuals(variant_source=vcf_file, case_lines=ped_lines)
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_individuals_gemini_database(self, gemini_path):
        individuals = get_individuals(variant_source=gemini_path, variant_mode='gemini')
        assert len(individuals) == 3
        ind_ids = set(['NA12878', 'NA12882','NA12877'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_individuals_from_vcf_no_ind(self, vcf_file_no_ind):
        individuals = get_individuals(variant_source=vcf_file_no_ind)
        assert len(individuals) == 0


class TestGetCase:
    
    def test_get_case_from_vcf(self, vcf_file):
        case_id = os.path.basename(vcf_file)
        case_obj = get_cases(vcf_file)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == False
        assert case_obj.tabix_index == False
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_case_no_ind(self, vcf_file_no_ind):
        case_id = os.path.basename(vcf_file_no_ind)
        case_obj = get_cases(vcf_file_no_ind)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == False
        assert case_obj.tabix_index == False
        
        individuals = case_obj.individuals
        assert len(individuals) == 0

    def test_get_case_from_compressed_vcf(self, compressed_vcf_file):
        case_id = os.path.basename(compressed_vcf_file)
        case_obj = get_cases(compressed_vcf_file)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == True
        assert case_obj.tabix_index == False
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_case_from_indexed_vcf(self, indexed_vcf_file):
        case_id = os.path.basename(indexed_vcf_file)
        case_obj = get_cases(indexed_vcf_file)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == True
        assert case_obj.tabix_index == True
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])
    
    def test_get_case_from_ped(self, vcf_file, ped_lines):
        case_id = '636808'
        case_obj = get_cases(vcf_file, case_lines=ped_lines)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == False
        assert case_obj.tabix_index == False
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])

    def test_get_case_from_ped_indexed_vcf(self, indexed_vcf_file, ped_lines):
        case_id = '636808'
        case_obj = get_cases(indexed_vcf_file, case_lines=ped_lines)[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == True
        assert case_obj.tabix_index == True
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['ADM1059A1','ADM1059A2','ADM1059A3'])
        assert ind_ids == set([ind.ind_id for ind in individuals])
    
    def test_get_case_from_gemini(self, gemini_path):
        case_id = '643594'
        case_obj = get_cases(gemini_path, variant_mode='gemini')[0]
        assert case_obj.case_id == case_id
        assert case_obj.compressed == False
        assert case_obj.tabix_index == False
        
        individuals = case_obj.individuals
        assert len(individuals) == 3
        ind_ids = set(['NA12878', 'NA12882','NA12877'])
        assert ind_ids == set([ind.ind_id for ind in individuals])
    


# -*- coding: utf-8 -*-
from datetime import datetime

from sqlalchemy import (Column, DateTime, ForeignKey, Integer, String, Boolean,
                        Text, UniqueConstraint)
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

from puzzle.models.mixins import PedigreeHumanMixin

# base for declaring a mapping
BASE = declarative_base()


class CaseIndividualLink(BASE):

    """Link between case and individuals."""

    __tablename__ = 'case_ind_link'
    __table_args__ = (UniqueConstraint('case_id', 'ind_id',
                                       name='_case_ind_uc'),)

    id = Column(Integer, primary_key=True)
    case_id = Column(Integer, ForeignKey('case.id'))
    ind_id = Column(Integer, ForeignKey('individual.id'))


class Case(BASE):
    """
    This is the class for storing cases in the database.

    A case will be related to Individuals
    """
    __tablename__ = "case"
    id = Column(Integer, primary_key=True)
    case_id = Column(String(32), unique=True)
    name = Column(String(32)) # This is the display name
    variant_source = Column(String)
    variant_type = Column(String) # snv or sv
    variant_mode = Column(String) # vcf or gemini
    pedigree = Column(String) # For storing madeline info
    compressed = Column(Boolean, default=False)
    tabix_index = Column(Boolean, default=False)
    synopsis = Column(Text)

    created_at = Column(DateTime, default=datetime.now)
    updated_at = Column(DateTime, default=datetime.now, onupdate=datetime.now)

    @property
    def suspect_dict(self):
        """Return all variant ids for pinned suspects."""
        return {suspect.variant_id: suspect for suspect in self.suspects}

    @property
    def case_comments(self):
        """Return only comments made on the case."""
        comments = (comment for comment in self.comments if
                    comment.variant_id is None)
        return comments

    @property
    def resources(self):
        """Return resources for all individuals."""
        for individual in self.individuals:
            for resource in individual.resources:
                yield resource

    @property
    def phenotypes(self):
        """Return all phenotypes for included individuals."""
        for individual in self.individuals:
            for term in individual.phenotypes:
                yield term

    def phenotype_ids(self):
        """Return only phenotype ids for included individuals."""
        return [term.phenotype_id for term in self.phenotypes]

    def __repr__(self):
        return("<Case(id:{self.id}, case_id:{self.case_id}, name:{self.name},"\
                " variant_source:{self.variant_source}, variant_type:"\
                "{self.variant_type}, variant_mode:{self.variant_mode}"\
                ", compressed:{self.compressed}, tabix_index:{self.tabix_index})"\
                ">".format(self=self))


class Individual(BASE, PedigreeHumanMixin):
    """
    This is the class for storing individuals in the database.

    A Individual belongs to a case
    """
    __tablename__ = "individual"
    id = Column(Integer, primary_key=True)
    ind_id = Column(String(32))
    name = Column(String(32))
    mother = Column(String(32))
    father = Column(String(32))
    sex = Column(String(32))
    phenotype = Column(String(32))
    ind_index = Column(Integer)
    variant_source = Column(String(32))
    bam_path = Column(String(32))
    cases = relationship('Case', secondary='case_ind_link', backref='individuals')

    def __repr__(self):
        return("<Individual(id:{self.id}, ind_id:{self.ind_id}, mother:{self.mother},"\
                " father{self.father}, sex:{self.sex}, phenotype.{self.phenotype}"\
                ", ind_index:{self.ind_index}, variant_source:"\
                "{self.variant_source})>".format(self=self))



from puzzle.plugins import GeminiPlugin

def test_get_individuals(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)
    
    ind_ids = [ind.ind_id for ind in adapter.individual_objs]
    assert set(ind_ids) == set(['NA12877', 'NA12878', 'NA12882'])

def test_get_individuals_one_ind(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    ind_ids = [ind.ind_id for ind in adapter.individuals('NA12877')]
    assert set(ind_ids) == set(['NA12877'])

def test_get_individuals_two_inds(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    ind_ids = [ind.ind_id for ind in adapter.individuals('NA12877', 'NA12878')]
    assert set(ind_ids) == set(['NA12877', 'NA12878'])

def test__get_individuals(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    ind_ids = [ind.ind_id for ind in adapter.individuals()]
    assert set(ind_ids) == set(['NA12877', 'NA12878', 'NA12882'])

def test_cases(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    case_ids = [case.case_id for case in adapter.cases()]
    assert set(case_ids) == set(['643594'])

def test_case_objs(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    case_ids = [case.case_id for case in adapter.case_objs]
    assert set(case_ids) == set(['643594'])

def test_case(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    case_id = '643594'
    assert adapter.case(case_id).case_id == case_id

def test_case_no_id(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    case_id = '643594'
    assert adapter.case().case_id == case_id

def test_case_wrong_id(gemini_case_obj):
    adapter = GeminiPlugin()
    adapter.add_case(gemini_case_obj)

    case_id = 'hello'
    assert adapter.case(case_id) == None



# -*- coding: utf-8 -*-
from .headers import (get_csq, get_header)
from .get_info import (get_most_severe_consequence, get_omim_number,
                       get_cytoband_coord, get_gene_info, get_gene_symbols)
from .ped import get_individuals, get_cases
from .phenomizer import hpo_genes
from .constants import IMPACT_SEVERITIES
from .get_file_info import (get_file_type, get_variant_type)


from . import (GeneExtras, FrequenciesExtras, ConsequenceExtras, 
               TranscriptExtras, GenotypeExtras, AnnotationExtras)

class VariantExtras(GeneExtras, FrequenciesExtras, ConsequenceExtras,
                    TranscriptExtras, GenotypeExtras, AnnotationExtras):
    """docstring for VariantExtras"""
    pass
        


# -*- coding: utf-8 -*-
import logging

from .mixins import PedigreeHumanMixin
from . import DotDict

logger = logging.getLogger(__name__)


class Gene(DotDict):
    """Class that holds information about a Gene"""
    def __init__(self, symbol, omim_number=None, ensembl_id=None, 
                description=None, chrom=None, start=None, stop=None,
                location=None, hi_score=None, constraint_score=None,
                hgnc_id=None):
        super(Gene, self).__init__(symbol=symbol, omim_number=omim_number,
        ensembl_id=ensembl_id, description=description, chrom=chrom, 
        start=start, stop=stop, location=location, hi_score=hi_score, 
        constraint_score=constraint_score, hgnc_id=hgnc_id)
        
        self['morbid'] = None



from .frequencies import FrequenciesExtras
from .genotypes import GenotypeExtras
from .transcripts import TranscriptExtras
from .consequences import ConsequenceExtras
from .genes import GeneExtras

from .core import VariantExtras


import logging

logger = logging.getLogger(__name__)

class FrequenciesExtras(object):
    """Methods for adding frequencies"""
    
    def _add_thousand_g(self, variant_obj, gemini_variant):
        """Add the thousand genomes frequency
        
        Args:
            variant_obj (puzzle.models.Variant)
            gemini_variant (GeminiQueryRow)
        """
        
        thousand_g = gemini_variant['aaf_1kg_all']
        if thousand_g:
            logger.debug("Updating thousand_g to: {0}".format(
                thousand_g))
            variant_obj.thousand_g = float(thousand_g)
            variant_obj.add_frequency('1000GAF', variant_obj.get('thousand_g'))
    
    def _add_gmaf(self, variant_obj, gemini_variant):
        """Add the gmaf frequency
        
        Args:
            variant_obj (puzzle.models.Variant)
            gemini_variant (GeminiQueryRow)
        
        """
        max_af = gemini_variant['max_aaf_all']
        if max_af:
            max_af = float(max_af)
            if max_af != -1.0:
                variant_obj.set_max_freq(max_af)

    def _add_exac(self, variant_obj, gemini_variant):
        """Add the gmaf frequency
        
        Args:
            variant_obj (puzzle.models.Variant)
            gemini_variant (GeminiQueryRow)        
        """
        exac = gemini_variant['aaf_exac_all']
        if exac:
            exac = float(exac)
            variant_obj.add_frequency('ExAC', exac)
            logger.debug("Updating ExAC to: {0}".format(
                exac))
    


# -*- coding: utf-8 -*-
from puzzle.models.case import Case


def test_Case():
    # with explicit name
    case_obj = Case('sample1', name='Oscar Wilde')
    assert case_obj.case_id == 'sample1'
    assert case_obj.name == 'Oscar Wilde'

    # without name
    case_obj = Case('sample2')
    assert case_obj.case_id == case_obj.name == 'sample2'



from puzzle.utils import (get_gene_info, get_cytoband_coord)


class BaseVariantMixin(object):
    """Base class for variant mixins"""
    
    def variants(self, case_id, skip=0, count=30, filters=None):
        """Return a results tuple with variants and nr_of_variants.

        """
        raise NotImplementedError

    def variant(self, variant_id):
        """Return a specific variant."""
        raise NotImplementedError
    
    def _get_genes(self, variant):
        """Add the genes for a variant

        Get the hgnc symbols from all transcripts and add them
        to the variant

        Args:
            variant (dict): A variant dictionary

        Returns:
            genes (list): A list of Genes
        """
        ensembl_ids = []
        hgnc_symbols = []
        
        for transcript in variant.transcripts:
            if transcript.ensembl_id:
                ensembl_ids.append(transcript.ensembl_id)
            if transcript.hgnc_symbol:
                hgnc_symbols.append(transcript.hgnc_symbol)
        
        genes = get_gene_info(
                        ensembl_ids=ensembl_ids, 
                        hgnc_symbols=hgnc_symbols
                        )
        return genes
    
    def _add_sv_coordinates(self, variant):
        """Add the neccesary sv coordinates for a variant
        
        Args:
            variant (puzzle.models.variant)
        """
        variant.stop_chrom = variant.CHROM
        variant.start = int(variant.POS)
        
        # If we have a translocation:
        if ':' in variant.ALT:
            other_coordinates = variant.ALT.strip('ACGTN[]').split(':')
            variant.stop_chrom = other_coordinates[0].lstrip('chrCHR')
            other_position = other_coordinates[1]
            # variant.stop = other_position

            #Set 'infinity' to length if translocation
            variant.sv_len = float('inf')
            variant.sv_type = 'BND'
        else:
            variant.sv_len = variant.stop - variant.start

        variant['cytoband_start'] = get_cytoband_coord(
                                        chrom=variant.CHROM,
                                        pos=variant.start
                                        )

        variant['cytoband_stop'] = get_cytoband_coord(
                                    chrom=variant.stop_chrom,
                                    pos=variant.stop
                                    )



import logging
import sqlite3

from contextlib import closing

from .constants import SCHEMA

logger = logging.getLogger(__name__)

def connect_database(db_path):
    """Connect to the event database"""
    logger.info("Connecting to database {0}".format(db_path))
    return sqlite3.connect(db_path)

def init_db(db_path):
    """Build the sqlite database"""
    logger.info("Creating database")
    with closing(connect_database(db_path)) as db:
        with open(SCHEMA, 'r') as f:
            db.cursor().executescript(f.read())
        db.commit()
    return


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
puzzle.__main__
~~~~~~~~~~~~~~~~~~~~~

The main entry point for the command line interface.

Invoke as ``puzzle`` (if installed)
or ``python -m puzzle`` (no install required).
"""
import sys

from puzzle.cli import base as cli


if __name__ == '__main__':
    # exit using whatever exit code the CLI returned
    sys.exit(cli())



# -*- coding: utf-8 -*-
from puzzle.plugins import VcfPlugin


def test_set_up(sql_store):
    assert sql_store.dialect == 'sqlite'


def test_select_plugin(test_db, case_obj):
    plugin, case_id = test_db.select_plugin(case_obj)
    assert isinstance(plugin, VcfPlugin)
    assert case_id == case_obj.case_id



# -*- coding: utf-8 -*-
from puzzle.models.sql import GeneList


def test_delete_gene():
    gene_list = GeneList(list_id='my cool list')
    assert gene_list.list_id == 'my cool list'
    assert gene_list.gene_ids == []

    gene_ids = ['ADK', 'SKD', 'EGFR']
    gene_list.gene_ids = gene_ids
    assert gene_list._gene_ids == ','.join(gene_ids)

    # remove one gene
    gene_list.delete_gene('ADK')
    assert gene_list.gene_ids == gene_ids[1:]

    # remove multiple genes
    gene_list.delete_gene('SKD', 'EGFR')
    assert gene_list.gene_ids == []



import os

import logging
import click

from . import (base, root)

from puzzle.plugins import SqlStore

logger = logging.getLogger(__name__)


@base.command()
@click.option('-f', '--family_id', type=str)
@click.option('-i', '--individual_id', type=str)
@root
@click.pass_context
def delete(ctx, family_id, individual_id, root):
    """
    Delete a case or individual from the database.

    If no database was found run puzzle init first.
    """
    root = root or ctx.obj.get('root') or os.path.expanduser("~/.puzzle")

    if os.path.isfile(root):
        logger.error("'root' can't be a file")
        ctx.abort()

    logger.info("Root directory is: {}".format(root))

    db_path = os.path.join(root, 'puzzle_db.sqlite3')
    logger.info("db path is: {}".format(db_path))

    if not os.path.exists(db_path):
        logger.warn("database not initialized, run 'puzzle init'")
        ctx.abort()

    store = SqlStore(db_path)

    if family_id:
        case_obj = store.case(case_id=family_id)
        if case_obj is None:
            logger.warning("Family {0} does not exist in database"
                           .format(family_id))
            ctx.abort()
        store.delete_case(case_obj)
    elif individual_id:
        ind_obj = store.individual(ind_id=individual_id)
        if ind_obj.ind_id != individual_id:
            logger.warning("Individual {0} does not exist in database"
                           .format(individual_id))
            ctx.abort()
        store.delete_individual(ind_obj)
    else:
        logger.warning("Please provide a family or individual id")
        ctx.abort()



from puzzle.models import (Transcript)
from gemini import GeminiQuery


class TranscriptExtras(object):
    """Collect the methods that deals with transcripts"""

    def _add_transcripts(self, variant_obj, gemini_variant):
        """
        Add all transcripts for a variant

        Go through all transcripts found for the variant

            Args:
                gemini_variant (GeminiQueryRow): The gemini variant

            Yields:
                transcript (puzzle.models.Transcript)

        """
        query = "SELECT * from variant_impacts WHERE variant_id = {0}".format(
            gemini_variant['variant_id']
        )

        gq = GeminiQuery(self.db)
        gq.run(query)
        
        for gemini_transcript in gq:
            transcript = Transcript(
                hgnc_symbol=gemini_transcript['gene'],
                transcript_id=gemini_transcript['transcript'],
                consequence=gemini_transcript['impact_so'],
                biotype=gemini_transcript['biotype'],
                polyphen=gemini_transcript['polyphen_pred'],
                sift=gemini_transcript['sift_pred'],
                HGVSc=gemini_transcript['codon_change'],
                HGVSp=', '.join([gemini_transcript['aa_change'] or '', gemini_transcript['aa_length'] or ''])
                )
            variant_obj.add_transcript(transcript)




import sys
import logging

from pprint import pprint as pp
from datetime import datetime
import click

from gemini import GeminiQuery
from gemini.gim import (AutoDom, AutoRec, DeNovo, MendelViolations, CompoundHet)

class Args(object):
    """
    >>> args = Args(db='some.db')
    >>> args.db
    'some.db'
    """
    _opts = ("columns", "db", "filter", "min_kindreds", "families",
                 "pattern_only", "max_priority", # only for compound_het
                 "allow_unaffected", "min_gq", "lenient", "min_sample_depth")
    def __init__(self, **kwargs):
        if not "min_gq" in kwargs: kwargs['min_gq'] = 0
        if not "lenient" in kwargs: kwargs['lenient'] = False
        for k in ("families", "filter"):
            if not k in kwargs: kwargs[k] = None
        if not "gt_phred_ll" in kwargs: kwargs['gt_phred_ll'] = None
        if not "min_sample_depth" in kwargs: kwargs['min_sample_depth'] = 0

        for k in ("min_kindreds", "max_priority"):
            if not k in kwargs: kwargs[k] = 1
        for k in ("pattern_only", "allow_unaffected"):
            if not k in kwargs: kwargs[k] = False
        self.__dict__.update(**kwargs)


def check_genotypes(gemini_variant, index):
    """Check the genotypes for a variant"""
    print(gemini_variant['gts'][index])
    print(gemini_variant['gt_types'][index])

@click.command()
@click.argument('db',
    type=click.Path(exists=True),
    # type=click.Path(exists=True),
)
@click.option('-v', '--variant',
    type=int
)
@click.option('--value',
)
@click.option('--query',
    default="SELECT * from variants"
)
def test_gemini(db, variant, value, query):
    """Test a gemini db"""
    gq = GeminiQuery(db)
    
    query = "SELECT * from variants"
    # query = "SELECT * from samples"
    
    # query = "gemini comp_hets"
    
    # if variant:
    #     query = "SELECT * from variants WHERE (aaf_1kg_all < {0} or aaf_1kg_all is Null)".format(variant)
    # else:
    #      query = "select * from variants"
    
    # query = "SELECT * from variants WHERE impact_severity in ('HIGH', 'LOW')"
    gq.run(query)
    # sample2idx = gq.sample_to_idx
    nr_of_variants = 0
    for index, variant in enumerate(gq):
        nr_of_variants += 1
    print(nr_of_variants)
    sys.exit()
    #     for key in variant.keys():
    #         if value:
    #             if key == value:
    #                 print("{0}: {1}".format(key, variant[key]))
    #         else:
    #             print("'{0}': {1},".format(key, variant[key]))
    #     print("")
        # check_genotypes(variant, 0)
        # check_genotypes(variant, 1)
        # check_genotypes(variant, 2)
    
    # print(sample2idx)
    # print(index)
    models = []
    #chrom,start,end,impact_severity,impact,gene,transcript,max_aaf_all
    models.append(AutoDom(Args(db=db,
                         columns="*")))
    models.append(AutoRec(Args(db=db,
                         columns="*")))
    models.append(DeNovo(Args(db=db,
                         columns="*",
                         families='643594')))
    models.append(CompoundHet(Args(db=db,
                         columns="*",
                         families='643594')))
    
    for model in models:
        variants = model.report_candidates()
        print(variants)
        print(model)
        nr_of_variants = 0
        for i, row in enumerate(variants):
            nr_of_variants += 1
            # print(i,row)
            # row is an OrderedDict
            # for key in row.keys():
            #     print("{0}:\t{1}".format(key, row[key]))
            # if i == 0:
            #     print "\t".join(row.keys())
            # print "\t".join(map(str, row.values()))
        print(nr_of_variants)
        print("")


if __name__ == '__main__':
    test_gemini()



from .genes import GeneExtras
from .frequencies import FrequenciesExtras
from .consequences import ConsequenceExtras
from .transcripts import TranscriptExtras
from .genotype import GenotypeExtras
from .annotations import AnnotationExtras
from .core import VariantExtras


# -*- coding: utf-8 -*-
"""
puzzle.plugins.sql.store
~~~~~~~~~~~~~~~~~~
"""
import logging
import os

from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

from puzzle.models.sql import BASE
from puzzle.plugins import VcfPlugin, Plugin

from .mixins import ActionsMixin, CaseMixin, VariantMixin

try:
    from puzzle.plugins import GeminiPlugin
except ImportError as e:
    pass

logger = logging.getLogger(__name__)


class Store(Plugin, CaseMixin, VariantMixin, ActionsMixin):

    """SQLAlchemy-based database object.

    .. note::
        For testing pourposes use ``:memory:`` as the ``path`` argument to
        set up in-memory (temporary) database.

    Args:
        uri (Optional[str]): path/URI to the database to connect to
        debug (Optional[bool]): whether to output logging information
        phenomizer_auth (Optional[tuple]): username and password

    Attributes:
        uri (str): path/URI to the database to connect to
        engine (class): SQLAlchemy engine, defines what database to use
        session (class): SQLAlchemy ORM session, manages persistance
        query (method): SQLAlchemy ORM query builder method
        classes (dict): bound ORM classes
    """

    def __init__(self, uri=None, debug=False, vtype='snv',
                 phenomizer_auth=None):
        super(Store, self).__init__()
        self.uri = uri
        if uri:
            self.connect(uri, debug=debug)
        self.variant_type = vtype
        self.phenomizer_auth = phenomizer_auth

        # ORM class shortcuts to enable fetching models dynamically
        # self.classes = {'gene': Gene, 'transcript': Transcript,
        #                 'exon': Exon, 'sample': Sample}

    def init_app(self, app):
        pass

    def connect(self, db_uri, debug=False):
        """Configure connection to a SQL database.

        Args:
            db_uri (str): path/URI to the database to connect to
            debug (Optional[bool]): whether to output logging information
        """
        kwargs = {'echo': debug, 'convert_unicode': True}
        # connect to the SQL database
        if 'mysql' in db_uri:
            kwargs['pool_recycle'] = 3600
        elif '://' not in db_uri:
            logger.debug("detected sqlite path URI: {}".format(db_uri))
            db_path = os.path.abspath(os.path.expanduser(db_uri))
            db_uri = "sqlite:///{}".format(db_path)

        self.engine = create_engine(db_uri, **kwargs)
        logger.debug('connection established successfully')
        # make sure the same engine is propagated to the BASE classes
        BASE.metadata.bind = self.engine
        # start a session
        self.session = scoped_session(sessionmaker(bind=self.engine))
        # shortcut to query method
        self.query = self.session.query
        return self

    @property
    def dialect(self):
        """Return database dialect name used for the current connection.
        Dynamic attribute.
        Returns:
            str: name of dialect used for database connection
        """
        return self.engine.dialect.name

    def set_up(self, reset=False):
        """Initialize a new database with the default tables and columns.
        Returns:
            Store: self
        """
        if reset:
            self.tear_down()

        logger.info("Creating database")
        # create the tables
        BASE.metadata.create_all(self.engine)
        return self

    def tear_down(self):
        """Tear down a database (tables and columns).
        Returns:
            Store: self
        """
        # drop/delete the tables
        logger.info('resetting database...')
        BASE.metadata.drop_all(self.engine)
        return self

    def save(self):
        """Manually persist changes made to various elements. Chainable.

        Returns:
            Store: ``self`` for chainability
        """
        # commit/persist dirty changes to the database
        self.session.flush()
        self.session.commit()
        return self

    def select_plugin(self, case_obj):
        """Select and initialize the correct plugin for the case."""
        if case_obj.variant_mode == 'vcf':
            logger.debug("Using vcf plugin")
            plugin = VcfPlugin(case_obj.variant_type)
        elif case_obj.variant_mode == 'gemini':
            logger.debug("Using gemini plugin")
            plugin = GeminiPlugin(case_obj.variant_type)
        
        #Add case to plugin
        plugin.add_case(case_obj)

        self.variant_type = case_obj.variant_type

        case_id = case_obj.case_id
        return plugin, case_id



# -*- coding: utf-8 -*-
import codecs
import json
import os
from pkg_resources import resource_filename
import gzip
from intervaltree import Interval, IntervalTree

import puzzle

SO_TERMS = (
  'transcript_ablation',
  'splice_donor_variant',
  'splice_acceptor_variant',
  'stop_gained',
  'frameshift_variant',
  'stop_lost',
  'start_lost',
  'initiator_codon_variant',
  'transcript_amplification',
  'inframe_insertion',
  'inframe_deletion',
  'missense_variant',
  'protein_altering_variant',
  'splice_region_variant',
  'incomplete_terminal_codon_variant',
  'stop_retained_variant',
  'synonymous_variant',
  'coding_sequence_variant',
  'mature_miRNA_variant',
  '5_prime_UTR_variant',
  '3_prime_UTR_variant',
  'non_coding_exon_variant',
  'non_coding_transcript_exon_variant',
  'non_coding_transcript_variant',
  'nc_transcript_variant',
  'intron_variant',
  'NMD_transcript_variant',
  'non_coding_transcript_variant',
  'upstream_gene_variant',
  'downstream_gene_variant',
  'TFBS_ablation',
  'TFBS_amplification',
  'TF_binding_site_variant',
  'regulatory_region_ablation',
  'regulatory_region_amplification',
  'regulatory_region_variant',
  'feature_elongation',
  'feature_truncation',
  'intergenic_variant'
)

IMPACT_SEVERITIES = {
    "transcript_ablation": "HIGH",
    "splice_acceptor_variant": "HIGH",
    "splice_donor_variant": "HIGH",
    "stop_gained": "HIGH",
    "stop_lost": "HIGH",
    "frameshift_variant": "HIGH",
    "start_lost": "HIGH",
    "exon_loss_variant": "HIGH",
    "initiator_codon_variant": "HIGH",
    "initiator_codon_variant": "HIGH",
    "chromosomal_deletion": "HIGH",
    "rare_amino_acid_variant": "HIGH",
    "missense_variant": "MEDIUM",
    "inframe_insertion": "MEDIUM",
    "inframe_deletion": "MEDIUM",
    "coding_sequence_variant": "MEDIUM",
    "disruptive_inframe_deletion": "MEDIUM",
    "disruptive_inframe_insertion": "MEDIUM",
    "5_prime_UTR_truncation": "MEDIUM",
    "exon_loss_variant": "MEDIUM",
    "3_prime_UTR_truncation": "MEDIUM",
    "exon_loss_variant": "MEDIUM",
    "splice_region_variant": "MEDIUM",
    "mature_miRNA_variant": "MEDIUM",
    "regulatory_region_variant": "MEDIUM",
    "TF_binding_site_variant": "MEDIUM",
    "regulatory_region_ablation": "MEDIUM",
    "regulatory_region_amplification": "MEDIUM",
    "TFBS_ablation": "MEDIUM",
    "TFBS_amplification": "MEDIUM",
    "stop_retained_variant": "LOW",
    "synonymous_variant": "LOW",
    "5_prime_UTR_variant": "LOW",
    "3_prime_UTR_variant": "LOW",
    "intron_variant": "LOW",
    "coding_sequence_variant": "LOW",
    "upstream_gene_variant": "LOW",
    "downstream_gene_variant": "LOW",
    "intergenic_variant": "LOW",
    "conserved_intergenic_variant": "LOW",
    "intragenic_variant": "LOW",
    "gene_variant": "LOW",
    "transcript_variant": "LOW",
    "exon_variant": "LOW",
    "5_prime_UTR_premature_start_codon_gain_variant": "LOW",
    "start_retained_variant": "LOW",
    "conserved_intron_variant": "LOW",
    "nc_transcript_variant": "LOW",
    "NMD_transcript_variant": "LOW",
    "incomplete_terminal_codon_variant": "LOW",
    "non_coding_exon_variant": "LOW",
    "transcript_amplification": "LOW",
    "feature_elongation": "LOW",
    "feature_truncation": "LOW",
}

# -*- coding: utf-8 -*-
SEVERITY_DICT = {}
for severity, term in enumerate(SO_TERMS):
    SEVERITY_DICT[term] = severity

resource_package = puzzle.__name__
hgnc_to_omim_path = os.path.join('utils', 'resources', 'hgnc_to_omim.json')
cytoband_path = os.path.join('utils', 'resources', 'cytoBand.txt.gz')

converter_path = resource_filename(resource_package, hgnc_to_omim_path)
cytoband_path = resource_filename(resource_package, cytoband_path)

with codecs.open(converter_path, 'r') as stream:
    HGNC_TO_OMIM = json.load(stream)

CYTOBANDS = {}

with gzip.open(cytoband_path, 'r') as cytobands:
    for line in cytobands:
        line = line.rstrip().split('\t')
        chrom = line[0].strip('chr')
        start = int(line[1])
        stop = int(line[2])
        cytoband = line[3]
        if chrom in CYTOBANDS:
            tree = CYTOBANDS[chrom]
            tree[start:stop] = cytoband
        else:
            CYTOBANDS[chrom] = IntervalTree([Interval(start, stop, cytoband)])



import logging

from puzzle.models import Genotype

logger = logging.getLogger(__name__)


class GenotypeExtras(object):
    """Class to store methods that deals with genotyping"""

    def _add_genotypes(self, variant_obj, gemini_variant, case_id,
                       individual_objs):
        """Add the genotypes for a variant for all individuals

                Args:
                    variant_obj (puzzle.models.Variant)
                    gemini_variant (GeminiQueryRow): The gemini variant
                    case_id (str): related case id
                    individual_objs (list(dict)): A list of Individuals

        """
        for ind in individual_objs:
            index = ind.ind_index
            variant_obj.add_individual(Genotype(
                sample_id=ind.ind_id,
                genotype=gemini_variant['gts'][index],
                case_id=case_id,
                phenotype=ind.phenotype,
                ref_depth=gemini_variant['gt_ref_depths'][index],
                alt_depth=gemini_variant['gt_alt_depths'][index],
                depth=gemini_variant['gt_depths'][index],
                genotype_quality=gemini_variant['gt_quals'][index]
            ))



# -*- coding: utf-8 -*-
from puzzle.plugins import VcfPlugin
from .blueprints import public_bp, variants_bp

PROJECT_NAME = __name__.split('.')[0]


class BaseConfig:
    PROJECT = PROJECT_NAME
    DEBUG = False
    TESTING = False

    # http://flask.pocoo.org/docs/quickstart/#sessions
    SECRET_KEY = 'secret key'
    PUZZLE_BACKEND = VcfPlugin()
    STORE_ENABLED = False

    # default blueprints
    BLUEPRINTS = [public_bp, variants_bp]

    #Phemonizer credentials
    PHENOMIZER_AUTH = False

class DevConfig(BaseConfig):
    DEBUG = True


class TestConfig(DevConfig):
    TESTING = True



# -*- coding: utf-8 -*-
import logging
import os

from ped_parser import FamilyParser
from ped_parser.exceptions import PedigreeError

from puzzle.models import Case, Individual

try:
    from gemini import GeminiQuery
except ImportError:
    pass

from . import get_header

logger = logging.getLogger(__name__)


def get_cases(variant_source, case_lines=None, case_type='ped',
              variant_type='snv', variant_mode='vcf'):
        """Create a cases and populate it with individuals

            Args:
                variant_source (str): Path to vcf files
                case_lines (Iterable): Ped like lines
                case_type (str): Format of case lines

            Returns:
                case_objs (list(puzzle.models.Case))
        """
        individuals = get_individuals(
            variant_source=variant_source,
            case_lines=case_lines,
            case_type=case_type,
            variant_mode=variant_mode
        )
        case_objs = []
        case_ids = set()

        compressed = False
        tabix_index = False
        #If no individuals we still need to have a case id
        if variant_source.endswith('.gz'):
            logger.debug("Found compressed variant source")
            compressed = True
            tabix_file = '.'.join([variant_source, 'tbi'])
            if os.path.exists(tabix_file):
                logger.debug("Found index file")
                tabix_index = True

        if len(individuals) > 0:
            for individual in individuals:
                case_ids.add(individual.case_id)
        else:
            case_ids = [os.path.basename(variant_source)]

        for case_id in case_ids:
            logger.info("Found case {0}".format(case_id))
            case = Case(
                case_id=case_id,
                name=case_id,
                variant_source=variant_source,
                variant_type=variant_type,
                variant_mode=variant_mode,
                compressed=compressed,
                tabix_index=tabix_index
                )

            # Add the individuals to the correct case
            for individual in individuals:
                if individual.case_id == case_id:
                    logger.info("Adding ind {0} to case {1}".format(
                        individual.name, individual.case_id
                    ))
                    case.add_individual(individual)

            case_objs.append(case)

        return case_objs


def get_individuals(variant_source, case_lines=None, case_type='ped', variant_mode='vcf'):
        """Get the individuals from a vcf file, gemini database, and/or a ped file.

            Args:
                variant_source (str): Path to a variant source
                case_lines(Iterable): Ped like lines
                case_type(str): Format of ped lines

            Returns:
                individuals (generator): generator with Individuals
        """
        individuals = []
        ind_dict ={}

        if variant_mode == 'vcf':
            head = get_header(variant_source)
            #Dictionary with ind_id:index where index show where in vcf ind info is

            for index, ind in enumerate(head.individuals):
                ind_dict[ind] = index

            if case_lines:
                # read individuals from ped file
                family_parser = FamilyParser(case_lines, family_type=case_type)
                families = family_parser.families
                logger.debug("Found families {0}".format(
                            ','.join(list(families.keys()))))
                if len(families) != 1:
                    logger.error("Only one family can be used with vcf adapter")
                    raise IOError

                case_id = list(families.keys())[0]
                logger.debug("Family used in analysis: {0}".format(case_id))

                for ind_id in family_parser.individuals:
                    ind = family_parser.individuals[ind_id]
                    logger.info("Found individual {0}".format(ind.individual_id))
                    try:
                        individual = Individual(
                            ind_id=ind_id,
                            case_id=case_id,
                            mother=ind.mother,
                            father=ind.father,
                            sex=str(ind.sex),
                            phenotype=str(ind.phenotype),
                            variant_source=variant_source,
                            ind_index=ind_dict[ind_id],
                            )
                        individuals.append(individual)
                    except KeyError as err:
                        #This is the case when individuals in ped does not exist
                        #in vcf
                        raise PedigreeError(
                            family_id=case_id,
                            individual_id=ind_id,
                            message="Individual {0} exists in ped file but not in vcf".format(ind_id)
                            )

            else:
                case_id = os.path.basename(variant_source)

                for ind in ind_dict:
                    individual = Individual(
                        ind_id=ind,
                        case_id=case_id,
                        variant_source=variant_source,
                        ind_index=ind_dict[ind]
                        )
                    individuals.append(individual)

                    logger.debug("Found individual {0} in {1}".format(
                                 ind, variant_source))
        elif variant_mode == 'gemini':
            gq = GeminiQuery(variant_source)
            #Dictionaru with sample to index in the gemini database
            ind_dict = gq.sample_to_idx
            query = "SELECT * from samples"
            gq.run(query)
            for individual in gq:
                logger.debug("Found individual {0} with family id {1}".format(
                    individual['name'], individual['family_id']))
                individuals.append(
                    Individual(
                        ind_id=individual['name'],
                        case_id=individual['family_id'],
                        mother=individual['maternal_id'],
                        father=individual['paternal_id'],
                        sex=individual['sex'],
                        phenotype=individual['phenotype'],
                        ind_index=ind_dict.get(individual['name']),
                        variant_source=variant_source,
                        bam_path=None)
                        )

        return individuals



# -*- coding: utf-8 -*-
import os
import logging

import yaml
import click

from puzzle.log import configure_stream, LEVELS
from puzzle.constants import PUZZLE_CONFIG_PATH

from . import (root, version, verbose)


logger = logging.getLogger(__name__)


@click.group()
@version
@verbose
@click.option('-c', '--config',
    type=click.Path(),
    default=PUZZLE_CONFIG_PATH
)
@click.pass_context
def base(ctx, verbose, config):
    """Puzzle: manage DNA variant resources."""
    # configure root logger to print to STDERR
    loglevel = LEVELS.get(min(verbose, 3))
    configure_stream(level=loglevel)
    ctx.obj = {}
    if config and os.path.exists(config):
        ctx.obj = yaml.load(open(config, 'r')) or {}
        ctx.obj['config_path'] = config
    # launch the command line interface
    logger.debug('Booting up command line interface')



import logging

logger = logging.getLogger(__name__)

class ConsequenceExtras(object):
    """Methods to handle consequences"""
    
    def _add_consequences(self, variant_obj):
        """Add the consequences found in all transcripts

        Args:
            variant_obj (puzzle.models.Variant)
        """

        consequences = set()
        for transcript in variant_obj.transcripts:
            for consequence in transcript.consequence.split('&'):
                consequences.add(consequence)

        variant_obj.consequences = list(consequences)
    
    def _add_most_severe_consequence(self, variant_obj, gemini_variant):
        """Add the most severe consequence
        
        Args:
            variant_obj (puzzle.models.Variant)
            gemini_variant (GeminiQueryRow)
        """
        variant_obj.most_severe_consequence = gemini_variant['impact_so']

    def _add_impact_severity(self, variant_obj, gemini_variant):
        """Add the impact severity for the most severe consequence
        
            Args:
                variant_obj (puzzle.models.Variant)
                gemini_variant (GeminiQueryRow)
        """
        gemini_impact = gemini_variant['impact_severity']
        if gemini_impact == 'MED':
            gemini_impact = 'MEDIUM'
        variant_obj.impact_severity = gemini_impact



##TODO mock a gemini db
import pytest
from puzzle.plugins import GeminiPlugin
from puzzle.models import Individual, Variant, DotDict

def test_get_all_variants(gemini_case_obj):
    """Test to get some variants from the gemini adapter"""
    plugin = GeminiPlugin()
    plugin.add_case(gemini_case_obj)

    filters = {}
    result = plugin.variants('643594', filters=filters, count=1000)
    variants = result.variants
    nr_of_variants = result.nr_of_variants

    assert nr_of_variants == 14

def test_get_variants(gemini_case_obj):
    """Test to get some variants from the gemini plugin"""
    plugin = GeminiPlugin()
    plugin.add_case(gemini_case_obj)

    filters = {}
    result = plugin.variants('643594', filters=filters, count=5)
    variants = result.variants
    nr_of_variants = result.nr_of_variants

    assert nr_of_variants == 5

def test_variant(gemini_case_obj):
    """Test to get one variant"""
    plugin = GeminiPlugin()
    plugin.add_case(gemini_case_obj)

    variant = plugin.variant(
        case_id='643594',
        variant_id=4
    )

    assert variant['CHROM'] == '6'
    assert variant['POS'] == '32487163'
    assert type(variant['genes']) == type([])

def test_is_variant(gemini_case_obj):
    plugin = GeminiPlugin()
    genotypes = ['G/A', 'G/A', 'G/G', 'G/G', './.']
    gt_types = [1, 1, 0, 0, 2]
    ind_objs = [
        Individual(ind_id=0, ind_index=0),
        Individual(ind_id=1, ind_index=1),
    ]
    #Mock a gemini variant
    gemini_variant = {
        'gts':genotypes,
        'gt_types':gt_types,
        'alt':'A'
    }

    assert plugin._is_variant(gemini_variant, ind_objs)

    #Check with individuals that are hom ref
    ind_objs = [
        Individual(ind_id=0, ind_index=2),
        Individual(ind_id=1, ind_index=3),
    ]

    assert not plugin._is_variant(gemini_variant, ind_objs)

    #Check with individual that has no call
    ind_objs = [
        Individual(ind_id=0, ind_index=4),
    ]

    assert not plugin._is_variant(gemini_variant, ind_objs)

def test_build_gemini_query():
    plugin = GeminiPlugin()
    query = "SELECT * from variants"
    extra_info = "max_aaf_all < 0.01"
    new_query = plugin.build_gemini_query(query, extra_info)
    assert new_query == "SELECT * from variants WHERE max_aaf_all < 0.01"

    extra_info = "cadd_score > 10"
    new_query = plugin.build_gemini_query(new_query, extra_info)
    assert new_query == "SELECT * from variants WHERE max_aaf_all < 0.01 AND cadd_score > 10"

class TestFilters:

    def test_filters_no_filters(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants

        assert nr_of_variants == 14

    def test_filters_frequency(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'frequency':'0.01'}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.max_freq < 0.01
        
        assert nr_of_variants == 13

    def test_filters_cadd(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'cadd':'20'}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.cadd_score > 20
        
        assert nr_of_variants == 4

    def test_filters_impact_severities_high(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'impact_severities':['HIGH']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'HIGH'
        
        assert nr_of_variants == 2

    def test_filters_impact_severities_medium(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'impact_severities':['MEDIUM']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'MEDIUM'
        assert nr_of_variants == 10

    def test_filters_impact_severities_low(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'impact_severities':['LOW']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity == 'LOW'
        
        assert nr_of_variants == 2

    def test_filters_impact_severities_high_and_med(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'impact_severities':['HIGH', 'MEDIUM']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.impact_severity in ['HIGH', 'MEDIUM']
        
        assert nr_of_variants == 12

    def test_filters_gene_ids(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'gene_ids':['HLA-DRB5']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert 'HLA-DRB5' in variant_obj.gene_symbols
        
        assert nr_of_variants == 5

    def test_filters_consequence(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        filters = {'consequence':['stop_gained']}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert 'stop_gained' in variant_obj.consequences
        
        assert nr_of_variants == 2

    def test_no_filters_sv(self, gemini_sv_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_sv_case_obj)

        filters = {}
        result = plugin.variants('hapmap', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        assert nr_of_variants == 513

    def test_filters_sv_len(self, gemini_sv_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_sv_case_obj)

        filters = {'sv_len':'800'}
        result = plugin.variants('hapmap', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.sv_len >= 800
        
        assert nr_of_variants == 176

    def test_filters_range(self, gemini_case_obj):
        plugin = GeminiPlugin()
        plugin.add_case(gemini_case_obj)

        start = 92498060
        end = 92498100

        filters = {'range': {'chromosome': 'chr1', 'start': start, 'end':end}}
        result = plugin.variants('643594', filters=filters, count=1000)
        variants = result.variants
        nr_of_variants = result.nr_of_variants
        
        for variant_obj in variants:
            assert variant_obj.start >= start
            assert variant_obj.stop <= end
        
        assert nr_of_variants == 1



import sys
import logging

from pprint import pprint as pp
from datetime import datetime
import click

from puzzle.utils import get_cytoband_coord

@click.command()
@click.option('-c','--chrom',
    type = str
)
@click.option('-p','--pos',
    type = int
)
def test_cytoband(chrom, pos):
    """Test to get cytoband coord"""
    if not (chrom and pos):
        print("Please provide coordinates")
        sys.exit()
        
    print(get_cytoband_coord(chrom, pos))

if __name__ == '__main__':
    test_cytoband()



# -*- coding: utf-8 -*-


def test_variants(test_db):
    # WHEN: there are some variants in the database for a case (vcf)

    # THEN: fetching variants for that case returnes a list of variant objs
    result = test_db.variants('636808')
    variant_objs = result.variants
    nr_of_variants = result.nr_of_variants
    assert nr_of_variants > 0
    assert variant_objs[0].start == 84563217

def test_variants_skip(test_db):

    # WHEN: skipping the first 10 variants
    result = test_db.variants('636808', skip=10)
    variant_objs = result.variants
    nr_of_variants = result.nr_of_variants

    # THEN: the first variant is the 11 overall
    assert variant_objs[0].start == 89185462

def test_variants_count(test_db):

    # WHEN: limiting the variants to 10 overall
    result = test_db.variants('636808', count=10)
    variant_objs = result.variants
    nr_of_variants = result.nr_of_variants

    # THEN: the the length of variants list is 10
    assert nr_of_variants == 10

def test_variant(test_db):
    # WHEN: fetching variant
    case_id = '636808'
    variant_id = '12_52696930_A_C'
    variant_obj = test_db.variant(case_id, variant_id)

    # THEN: variant with position '52696930' is returned
    assert variant_obj.start == 52696929

def test_non_existing_variant(test_db):
    # WHEN: fetching variant
    case_id = '636808'
    variant_id = '1_880086_T_C'
    variant_obj = test_db.variant(case_id, variant_id)

    # THEN: variant with position '880086' is returned
    assert variant_obj == None



from puzzle.plugins import VcfPlugin

def test_add_consequences(variant):
    plugin = VcfPlugin()
    # The raw variant line is treated like a string so it does not matter
    # if it is malformed
    raw_variant_line = "7\t98994144\t.\tA\tC\t51.68\tPASS\tAC=1;AF=0.167;AN=6"\
    ";BaseQRankSum=-1.965;CADD=13.51;CSQ=C|downstream_gene_variant|MODIFIER|"\
    "ARPC1B|ENSG00000130429|Transcript|ENST00000491294|retained_intron|||||||"\
    "||||1724|1|HGNC|704||||||||||||||,C|downstream_gene_variant|MODIFIER|"\
    "ARPC1B|ENSG00000130429|Transcript|ENST00000451682|protein_coding||||||||"\
    "|||1745|1|HGNC|704||CCDS5661.1|ENSP00000389631|ARC1B_HUMAN|F8VXW2_HUMAN"\
    "&C9K057_HUMAN&C9JTT6_HUMAN&C9JQM8_HUMAN&C9JM51_HUMAN&C9JFG9_HUMAN&C9JEY1"\
    "_HUMAN&C9JBJ7_HUMAN&C9J6C8_HUMAN&C9J4Z7_HUMAN&A4D275_HUMAN|UPI0000125D33|"
    
    plugin._add_consequences(variant, raw_variant_line)
    
    assert variant.consequences == ["downstream_gene_variant"]

def test_add_most_severe_no_info(variant):
    plugin = VcfPlugin()
    plugin._add_most_severe_consequence(variant)
    
    assert variant.most_severe_consequence == None

def test_add_most_severe(variant):
    plugin = VcfPlugin()
    
    variant.consequences = ['downstream_gene_variant', 'stop_gained']
    plugin._add_most_severe_consequence(variant)
    
    assert variant.most_severe_consequence == 'stop_gained'

def test_add_impact_severity(variant):
    plugin = VcfPlugin()
    
    variant.most_severe_consequence = 'stop_gained'
    plugin._add_impact_severity(variant)
    
    assert variant.impact_severity == 'HIGH'


# -*- coding: utf-8 -*-
from flask import (abort, current_app as app, Blueprint, render_template,
                   request, redirect)

from puzzle._compat import iteritems
from puzzle.constants import (INHERITANCE_MODELS_SHORT, SO_TERMS, SV_TYPES,
                              IMPACT_LEVELS)

BP_NAME = __name__.split('.')[-2]
blueprint = Blueprint(BP_NAME, __name__, url_prefix='/variants',
                      template_folder='templates', static_folder='static',
                      static_url_path="/{}/static".format(BP_NAME))


@blueprint.route('/<case_id>')
def variants(case_id):
    """Show all variants for a case."""
    filters = parse_filters()
    values = [value for key, value in iteritems(filters)
              if not isinstance(value, dict) and key != 'skip']
    is_active = any(values)
    variants, nr_of_variants = app.db.variants(
        case_id,
        skip=filters['skip'],
        filters={
            'gene_ids': filters['gene_symbols'],
            'frequency': filters.get('frequency'),
            'cadd': filters.get('cadd'),
            'sv_len': filters.get('sv_len'),
            'consequence': filters['selected_consequences'],
            'genetic_models': filters['selected_models'],
            'sv_types': filters['selected_sv_types'],
            'gene_lists': filters['gene_lists'],
            'impact_severities': filters['impact_severities'],
            'gemini_query': filters['gemini_query'],
            'range': filters['range'],
        }
    )
    gene_lists = ([gene_list.list_id for gene_list in app.db.gene_lists()]
                  if app.config['STORE_ENABLED'] else [])
    queries = ([(query.name or query.query, query.query) for query
                in app.db.gemini_queries()]
               if app.config['STORE_ENABLED'] else [])
    kwargs = dict(variants=variants, case_id=case_id, db=app.db,
                  filters=filters, consequences=SO_TERMS,
                  inheritance_models=INHERITANCE_MODELS_SHORT,
                  gene_lists=gene_lists, impact_severities=IMPACT_LEVELS,
                  is_active=is_active, nr_of_variants=nr_of_variants,
                  queries=queries)

    if app.db.variant_type == 'sv':
        return render_template('sv_variants.html', sv_types=SV_TYPES, **kwargs)
    else:
        return render_template('variants.html', **kwargs)


@blueprint.route('/<case_id>/<variant_id>')
def variant(case_id, variant_id):
    """Show a single variant."""
    case_obj = app.db.case(case_id)
    variant = app.db.variant(case_id, variant_id)
    if variant is None:
        return abort(404, "variant not found")

    comments = app.db.comments(variant_id=variant.md5)
    template = 'sv_variant.html' if app.db.variant_type == 'sv' else 'variant.html'
    return render_template(template, variant=variant, case_id=case_id,
                           comments=comments, case=case_obj)


def parse_filters():
    """Parse variant filters from the request object."""
    genes_str = request.args.get('gene_symbol')
    filters = {}
    for key in ('frequency', 'cadd', 'sv_len'):
        try:
            filters[key] = float(request.args.get(key))
        except (ValueError, TypeError):
            pass

    filters['gene_symbols'] = genes_str.split(',') if genes_str else None
    filters['selected_models'] = request.args.getlist('inheritance_models')
    filters['selected_consequences'] = request.args.getlist('consequences')
    filters['selected_sv_types'] = request.args.getlist('sv_types')
    filters['skip'] = int(request.args.get('skip', 0))
    filters['gene_lists'] = request.args.getlist('gene_lists')
    filters['gemini_query'] = (request.args.get('gemini_query') or
                               request.args.get('preset_gemini_query'))
    filters['impact_severities'] = request.args.getlist('impact_severities')
    filters['range'] = None

    if request.args.get('range'):
        chromosome, raw_pos = request.args.get('range').split(':')
        start, end = map(int, raw_pos.split('-'))
        filters['range'] = {'chromosome': chromosome, 'start': start,
                            'end': end}

    filters['query_dict'] = {key: request.args.getlist(key) for key
                             in request.args.keys()}
    filters['query_dict'].update({'skip': (filters['skip'] + 30)})

    return filters


@blueprint.route('/<case_id>/suspects/<variant_id>', methods=['POST'])
def suspects(case_id, variant_id):
    """Pin a variant as a suspect for a given case."""
    case_obj = app.db.case(case_id)
    variant_obj = app.db.variant(case_id, variant_id)
    app.db.add_suspect(case_obj, variant_obj)
    return redirect(request.referrer)


@blueprint.route('/suspects/<suspect_id>', methods=['POST'])
def delete_suspect(suspect_id):
    """Delete a suspect."""
    app.db.delete_suspect(suspect_id)
    return redirect(request.referrer)


@blueprint.route('/queries', methods=['POST'])
def queries():
    """Store a new GEMINI query."""
    query = request.form['query']
    name = request.form.get('name')
    app.db.add_gemini_query(name, query)
    return redirect(request.referrer)


@blueprint.route('/queries/delete/<query_id>', methods=['POST'])
def delete_query(query_id):
    """Delete a query."""
    app.db.delete_gemini_query(query_id)
    return redirect(request.referrer)



# -*- coding: utf-8 -*-
from sqlalchemy import Column, ForeignKey, Integer, String, UniqueConstraint
from sqlalchemy.orm import relationship

from .models import BASE


class CaseGenelistLink(BASE):

    """Link between case and gene list."""

    __tablename__ = 'case_genelist_link'
    __table_args__ = (UniqueConstraint('case_id', 'genelist_id',
                                       name='_case_genelist_uc'),)

    id = Column(Integer, primary_key=True)
    case_id = Column(Integer, ForeignKey('case.id'))
    genelist_id = Column(Integer, ForeignKey('gene_list.id'))


class GeneList(BASE):

    """Represent a list of gene identifiers."""

    __tablename__ = "gene_list"

    id = Column(Integer, primary_key=True)
    list_id = Column(String(32), nullable=False, unique=True)
    # comma separated list of gene ids
    _gene_ids = Column(String(1024))

    cases = relationship('Case', secondary='case_genelist_link',
                         backref='gene_lists')

    @property
    def gene_ids(self):
        """Return a list of gene ids."""
        return self._gene_ids.split(',') if self._gene_ids else []

    @gene_ids.setter
    def gene_ids(self, value):
        self._gene_ids = ','.join(value)

    def delete_gene(self, *gene_ids):
        """Delete one or more gene ids form the list."""
        self.gene_ids = [gene_id for gene_id in self.gene_ids
                         if gene_id not in gene_ids]

    def __repr__(self):
        return "PhenotypeTerm(list_id={this.list_id})".format(this=self)



from vcftoolbox import (get_vep_info, get_snpeff_info)

from puzzle.models import (Transcript)


class TranscriptExtras(object):
    """Collect the methods that deals with transcripts"""
    
    def _add_transcripts(self, variant_obj, info_dict):
        """Return all transcripts sound in the vcf file"""
        vep_string = info_dict.get('CSQ')

        #Check if snpeff annotation:
        snpeff_string = info_dict.get('ANN')
        
        # We check one of these.
        # VEP has presedence over snpeff
        if vep_string:
            #Get the vep annotations
            vep_info = get_vep_info(
                vep_string = vep_string,
                vep_header = self.vep_header
                )
            for transcript_info in vep_info:
                transcript = self._get_vep_transcript(transcript_info)
                variant_obj.add_transcript(transcript)

        elif snpeff_string:
            #Get the vep annotations
            snpeff_info = get_snpeff_info(
                snpeff_string = snpeff_string,
                snpeff_header = self.snpeff_header
                )
            for transcript_info in snpeff_info:
                transcript = self._get_snpeff_transcript(transcript_info)
                variant_obj.add_transcript(transcript)

    
    def _get_vep_transcript(self, transcript_info):
        """Create a Transcript based on the vep annotation

            Args:
                transcript_info (dict): A dict with vep info

            Returns:
                transcript (puzzle.models.Transcript): A Transcripts
        """
        transcript = Transcript(
                hgnc_symbol = transcript_info.get('SYMBOL'),
                transcript_id = transcript_info.get('Feature'),
                ensembl_id = transcript_info.get('Gene'),
                biotype = transcript_info.get('BIOTYPE'),
                consequence = transcript_info.get('Consequence'),
                strand = transcript_info.get('STRAND'),
                sift = transcript_info.get('SIFT'),
                polyphen = transcript_info.get('PolyPhen'),
                exon = transcript_info.get('EXON'),
                HGVSc = transcript_info.get('HGVSc'),
                HGVSp = transcript_info.get('HGVSp'),
                GMAF = transcript_info.get('GMAF'),
                ExAC_MAF = transcript_info.get('ExAC_MAF')
            )
        return transcript

    def _get_snpeff_transcript(self, transcript_info):
        """Create a transcript based on the snpeff annotation

            Args:
                transcript_info (dict): A dict with snpeff info

            Returns:
                transcript (puzzle.models.Transcript): A Transcripts
        """
        transcript = Transcript(
                hgnc_symbol = transcript_info.get('Gene_Name'),
                transcript_id = transcript_info.get('Feature'),
                ensembl_id = transcript_info.get('Gene_ID'),
                biotype = transcript_info.get('Transcript_BioType'),
                consequence = transcript_info.get('Annotation'),
                exon = transcript_info.get('Rank'),
                HGVSc = transcript_info.get('HGVS.c'),
                HGVSp = transcript_info.get('HGVS.p')
            )
        return transcript
    